import os.path
import re
import sys
import unittest

from robot.running.testlibraries import (TestLibrary, _ClassLibrary,
                                         _ModuleLibrary, _DynamicLibrary)
from robot.utils.asserts import (assert_equal, assert_false, assert_none,
                                 assert_not_equal, assert_not_none, assert_true,
                                 assert_raises, assert_raises_with_msg)
from robot.utils import normalize
from robot.errors import DataError

from classes import (NameLibrary, DocLibrary, ArgInfoLibrary, GetattrLibrary,
                     SynonymLibrary, __file__ as classes_source)


# Valid keyword names and arguments for some libraries
default_keywords = [ ( "no operation", () ),
                     ( "log", ("msg",) ),
                     ( "L O G", ("msg","warning") ),
                     ( "fail", () ),
                     ( "  f  a  i  l  ", ("msg",) ) ]
example_keywords = [ ( "Log", ("msg",) ),
                     ( "log many", () ),
                     ( "logmany", ("msg",) ),
                     ( "L O G M A N Y", ("m1","m2","m3","m4","m5") ),
                     ( "equals", ("1","1") ),
                     ( "equals", ("1","2","failed") ), ]


class TestLibraryTypes(unittest.TestCase):

    def test_python_library(self):
        lib = TestLibrary("BuiltIn")
        assert_equal(lib.__class__, _ClassLibrary)
        assert_equal(lib.positional_args, [])

    def test_python_library_with_args(self):
        lib = TestLibrary("ParameterLibrary", ['my_host', '8080'])
        assert_equal(lib.__class__, _ClassLibrary)
        assert_equal(lib.positional_args, ['my_host', '8080'])

    def test_module_library(self):
        lib = TestLibrary("module_library")
        assert_equal(lib.__class__, _ModuleLibrary)

    def test_module_library_with_args(self):
        assert_raises(DataError, TestLibrary, "module_library", ['arg'] )

    def test_dynamic_python_library(self):
        lib = TestLibrary("RunKeywordLibrary")
        assert_equal(lib.__class__, _DynamicLibrary)


class TestImports(unittest.TestCase):

    def test_import_python_class(self):
        lib = TestLibrary("BuiltIn")
        self._verify_lib(lib, "BuiltIn", default_keywords)

    def test_import_python_class_from_module(self):
        lib = TestLibrary("robot.libraries.BuiltIn.BuiltIn")
        self._verify_lib(lib, "robot.libraries.BuiltIn.BuiltIn", default_keywords)

    def test_import_python_module(self):
        lib = TestLibrary("module_library")
        kws = ["passing", "two arguments from class", "lambdakeyword", "argument"]
        self._verify_lib(lib, "module_library", [(kw, None) for kw in kws])

    def test_import_python_module_from_module(self):
        lib = TestLibrary("pythonmodule.library")
        self._verify_lib(lib, "pythonmodule.library",
                         [("keyword from submodule", None)])

    def test_import_non_existing_module(self):
        msg = ("Importing library '{libname}' failed: "
               "ModuleNotFoundError: No module named '{modname}'")
        for name in 'nonexisting', 'nonexi.sting':
            error = assert_raises(DataError, TestLibrary, name)
            expected = msg.format(libname=name, modname=name.split('.')[0])
            assert_equal(str(error).splitlines()[0], expected)

    def test_import_non_existing_class_from_existing_module(self):
        assert_raises_with_msg(DataError,
                               "Importing library 'pythonmodule.NonExisting' failed: "
                               "Module 'pythonmodule' does not contain 'NonExisting'.",
                               TestLibrary, 'pythonmodule.NonExisting')

    def test_import_invalid_type(self):
        msg = "Importing library '%s' failed: Expected class or module, got %s."
        assert_raises_with_msg(DataError,
                               msg % ('pythonmodule.some_string', 'string'),
                               TestLibrary, 'pythonmodule.some_string')
        assert_raises_with_msg(DataError,
                               msg % ('pythonmodule.some_object', 'SomeObject'),
                               TestLibrary, 'pythonmodule.some_object')

    def test_import_with_unicode_name(self):
        self._verify_lib(TestLibrary(u"BuiltIn"), "BuiltIn", default_keywords)
        self._verify_lib(TestLibrary(u"robot.libraries.BuiltIn.BuiltIn"),
                         "robot.libraries.BuiltIn.BuiltIn", default_keywords)
        self._verify_lib(TestLibrary(u"pythonmodule.library"), "pythonmodule.library",
                         [("keyword from submodule", None)])

    def test_global_scope(self):
        self._verify_scope(TestLibrary('libraryscope.Global'), 'GLOBAL')

    def _verify_scope(self, lib, expected):
        assert_equal(str(lib.scope), expected)

    def test_suite_scope(self):
        self._verify_scope(TestLibrary('libraryscope.Suite'), 'SUITE')
        self._verify_scope(TestLibrary('libraryscope.TestSuite'), 'SUITE')

    def test_test_scope(self):
        self._verify_scope(TestLibrary('libraryscope.Test'), 'TEST')
        self._verify_scope(TestLibrary('libraryscope.TestCase'), 'TEST')

    def test_task_scope_is_mapped_to_test_scope(self):
        self._verify_scope(TestLibrary('libraryscope.Task'), 'TEST')

    def test_invalid_scope_is_mapped_to_test_scope(self):
        for libname in ['libraryscope.InvalidValue',
                        'libraryscope.InvalidEmpty',
                        'libraryscope.InvalidMethod',
                        'libraryscope.InvalidNone']:
            self._verify_scope(TestLibrary(libname), 'TEST')

    def _verify_lib(self, lib, libname, keywords):
        assert_equal(libname, lib.name)
        for name, _ in keywords:
            handler = lib.handlers[name]
            exp = "%s.%s" % (libname, name)
            assert_equal(normalize(handler.longname), normalize(exp))


class TestLibraryInit(unittest.TestCase):

    def test_python_library_without_init(self):
        self._test_init_handler('ExampleLibrary')

    def test_python_library_with_init(self):
        self._test_init_handler('ParameterLibrary', ['foo'], 0, 2)

    def test_new_style_class_without_init(self):
        self._test_init_handler('newstyleclasses.NewStyleClassLibrary')

    def test_new_style_class_with_init(self):
        lib = self._test_init_handler('newstyleclasses.NewStyleClassArgsLibrary', ['value'], 1, 1)
        assert_equal(len(lib.handlers), 1)

    def test_library_with_metaclass(self):
        self._test_init_handler('newstyleclasses.MetaClassLibrary')

    def test_library_with_zero_len(self):
        self._test_init_handler('LenLibrary')

    def _test_init_handler(self, libname, args=None, min=0, max=0):
        lib = TestLibrary(libname, args)
        assert_equal(lib.init.arguments.minargs, min)
        assert_equal(lib.init.arguments.maxargs, max)
        return lib


class TestVersion(unittest.TestCase):

    def test_no_version(self):
        self._verify_version('classes.NameLibrary', '')

    def test_version_in_class_library(self):
        self._verify_version('classes.VersionLibrary', '0.1')
        self._verify_version('classes.VersionObjectLibrary', 'ver')

    def test_version_in_module_library(self):
        self._verify_version('module_library', 'test')

    def _verify_version(self, name, version):
        assert_equal(TestLibrary(name).version, version)


class TestDocFormat(unittest.TestCase):

    def test_no_doc_format(self):
        self._verify_doc_format('classes.NameLibrary', '')

    def test_doc_format_in_python_libarary(self):
        self._verify_doc_format('classes.VersionLibrary', 'HTML')

    def _verify_doc_format(self, name, doc_format):
        assert_equal(TestLibrary(name).doc_format, doc_format)


class _TestScopes(unittest.TestCase):

    def _get_lib_and_instance(self, name):
        lib = TestLibrary(name)
        if lib.scope.is_global:
            assert_not_none(lib._libinst)
        else:
            assert_none(lib._libinst)
        return lib, lib._libinst

    def _start_new_suite(self):
        self.lib.start_suite()
        assert_none(self.lib._libinst)
        inst = self.lib.get_instance()
        assert_not_none(inst)
        return inst

    def _verify_end_suite_restores_previous_instance(self, prev_inst):
        self.lib.end_suite()
        assert_true(self.lib._libinst is prev_inst)
        if prev_inst is not None:
            assert_true(self.lib.get_instance() is prev_inst)


class GlobalScope(_TestScopes):

    def test_global_scope(self):
        lib, instance = self._get_lib_and_instance('BuiltIn')
        for mname in ['start_suite', 'start_suite', 'start_test', 'end_test',
                      'start_test', 'end_test', 'end_suite', 'start_suite',
                      'start_test', 'end_test', 'end_suite', 'end_suite']:
            getattr(lib, mname)()
            assert_true(instance is lib._libinst)


class TestSuiteScope(_TestScopes):

    def setUp(self):
        self.lib, self.instance = self._get_lib_and_instance("libraryscope.Suite")
        self.lib.start_suite()

    def test_start_suite_flushes_instance(self):
        assert_none(self.lib._libinst)
        inst = self.lib.get_instance()
        assert_not_none(inst)
        assert_false(inst is self.instance)

    def test_start_test_or_end_test_do_not_flush_instance(self):
        inst = self.lib.get_instance()
        for _ in range(10):
            self.lib.start_test()
            assert_true(inst is self.lib._libinst)
            assert_true(inst is self.lib.get_instance())
            self.lib.end_test()
            assert_true(inst is self.lib._libinst)

    def test_end_suite_restores_previous_instance_with_one_suite(self):
        self.lib.start_test()
        self.lib.get_instance()
        self.lib.end_test()
        self.lib.get_instance()
        self.lib.end_suite()
        assert_none(self.lib._libinst)

    def test_intance_caching(self):
        inst1 = self.lib.get_instance()
        inst2 = self._start_new_suite()
        assert_false(inst1 is inst2)
        self._run_tests(inst2)
        self._verify_end_suite_restores_previous_instance(inst1)
        inst3 = self._start_new_suite()
        inst4 = self._start_new_suite()
        self._run_tests(inst4, 10)
        self._verify_end_suite_restores_previous_instance(inst3)
        self._verify_end_suite_restores_previous_instance(inst1)
        self._verify_end_suite_restores_previous_instance(None)

    def _run_tests(self, exp_inst, count=3):
        for _ in range(count):
            self.lib.start_test()
            assert_true(self.lib.get_instance() is exp_inst)
            self.lib.end_test()
            assert_true(self.lib.get_instance() is exp_inst)


class TestCaseScope(_TestScopes):

    def setUp(self):
        self.lib, self.instance = self._get_lib_and_instance("libraryscope.Test")
        self.lib.start_suite()

    def test_different_instances_for_all_tests(self):
        self._run_tests(None)
        inst = self.lib.get_instance()
        self._run_tests(inst, 5)
        self.lib.end_suite()
        assert_none(self.lib._libinst)

    def test_nested_suites(self):
        top_inst = self.lib.get_instance()
        self._run_tests(top_inst, 4)
        self.lib.start_suite()
        self._run_tests(None, 3)
        self.lib.start_suite()
        self._run_tests(self.lib.get_instance(), 3)
        self.lib.end_suite()
        self.lib.end_suite()
        assert_true(self.lib._libinst is top_inst)

    def _run_tests(self, suite_inst, count=3):
        old_insts = [suite_inst]
        for _ in range(count):
            self.lib.start_test()
            assert_none(self.lib._libinst)
            inst = self.lib.get_instance()
            assert_false(inst in old_insts)
            old_insts.append(inst)
            self.lib.end_test()
            assert_true(self.lib._libinst is suite_inst)


class TestHandlers(unittest.TestCase):

    def test_get_handlers(self):
        for lib in [NameLibrary, DocLibrary, ArgInfoLibrary, GetattrLibrary, SynonymLibrary]:
            handlers = TestLibrary('classes.%s' % lib.__name__).handlers
            assert_equal(lib.handler_count, len(handlers), lib.__name__)
            for handler in handlers:
                assert_false(handler._handler_name.startswith('_'))
                assert_true('skip' not in handler._handler_name)

    def test_non_global_dynamic_handlers(self):
        lib = TestLibrary("RunKeywordLibrary")
        assert_equal(len(lib.handlers), 2)
        assert_true('Run Keyword That Passes' in lib.handlers)
        assert_true('Run Keyword That Fails' in lib.handlers)
        assert_none(lib.handlers['Run Keyword That Passes']._method)
        assert_none(lib.handlers['Run Keyword That Fails']._method)

    def test_global_dynamic_handlers(self):
        lib = TestLibrary("RunKeywordLibrary.GlobalRunKeywordLibrary")
        assert_equal(len(lib.handlers), 2)
        for name in 'Run Keyword That Passes', 'Run Keyword That Fails':
            handler = lib.handlers[name]
            assert_not_none(handler._method)
            assert_not_equal(handler._method, lib._libinst.run_keyword)
            assert_equal(handler._method.__name__, 'handler')

    def test_synonym_handlers(self):
        testlib = TestLibrary('classes.SynonymLibrary')
        names = ['handler', 'synonym_handler', 'another_synonym']
        for handler in testlib.handlers:
            # test 'handler_name' -- raises ValueError if it isn't in 'names'
            names.remove(handler._handler_name)
        assert_equal(len(names), 0, 'handlers %s not created' % names, False)

    def test_global_handlers_are_created_only_once(self):
        lib = TestLibrary('classes.RecordingLibrary')
        assert_true(lib.scope.is_global)
        instance = lib._libinst
        assert_true(instance is not None)
        assert_equal(instance.kw_accessed, 1)
        assert_equal(instance.kw_called, 0)
        for _ in range(5):
            lib.handlers['kw'].create_runner('kw')._run(_FakeContext(), [])
        assert_true(lib._libinst is instance)
        assert_equal(instance.kw_accessed, 1)
        assert_equal(instance.kw_called, 5)


class TestDynamicLibrary(unittest.TestCase):

    def test_get_keyword_doc_is_used_if_present(self):
        lib = TestLibrary('classes.ArgDocDynamicLibrary')
        assert_equal(lib.handlers['No Arg'].doc,
                     'Keyword documentation for No Arg')
        assert_equal(lib.handlers['Multiline'].doc,
                     'Multiline\nshort doc!\n\nBody\nhere.')

    def test_get_keyword_doc_and_args_are_ignored_if_not_callable(self):
        lib = TestLibrary('classes.InvalidAttributeDynamicLibrary')
        assert_equal(len(lib.handlers), 7)
        assert_equal(lib.handlers['No Arg'].doc, '')
        assert_handler_args(lib.handlers['No Arg'], 0, sys.maxsize)

    def test_handler_is_not_created_if_get_keyword_doc_fails(self):
        lib = TestLibrary('classes.InvalidGetDocDynamicLibrary')
        assert_equal(len(lib.handlers), 0)

    def test_handler_is_not_created_if_get_keyword_args_fails(self):
        lib = TestLibrary('classes.InvalidGetArgsDynamicLibrary')
        assert_equal(len(lib.handlers), 0)

    def test_arguments_without_kwargs(self):
        lib = TestLibrary('classes.ArgDocDynamicLibrary')
        for name, (mina, maxa) in [('No Arg', (0, 0)),
                                   ('One Arg', (1, 1)),
                                   ('One or Two Args', (1, 2)),
                                   ('Many Args', (0, sys.maxsize)),
                                   ('No Arg Spec', (0, sys.maxsize))]:
            assert_handler_args(lib.handlers[name], mina, maxa)

    def test_arguments_with_kwargs(self):
        lib = TestLibrary('classes.ArgDocDynamicLibraryWithKwargsSupport')
        for name, (mina, maxa) in [('No Arg', (0, 0)),
                                   ('One Arg', (1, 1)),
                                   ('One or Two Args', (1, 2)),
                                   ('Many Args', (0, sys.maxsize))]:
            assert_handler_args(lib.handlers[name], mina, maxa, kwargs=False)
        for name, (mina, maxa) in [('Kwargs', (0, 0)),
                                   ('Varargs and Kwargs', (0, sys.maxsize)),
                                   ('No Arg Spec', (0, sys.maxsize))]:
            assert_handler_args(lib.handlers[name], mina, maxa, kwargs=True)


def assert_handler_args(handler, minargs=0, maxargs=0, kwargs=False):
    assert_equal(handler.arguments.minargs, minargs)
    assert_equal(handler.arguments.maxargs, maxargs)
    assert_equal(bool(handler.arguments.var_named), kwargs)


class TestDynamicLibraryIntroDocumentation(unittest.TestCase):

    def test_doc_from_class_definition(self):
        self._assert_intro_doc('dynlibs.StaticDocsLib',
                               'This is lib intro.')

    def test_doc_from_dynamic_method(self):
        self._assert_intro_doc('dynlibs.DynamicDocsLib',
                               'Dynamic intro doc.')

    def test_dynamic_doc_overrides_class_doc(self):
        self._assert_intro_doc('dynlibs.StaticAndDynamicDocsLib',
                                         'dynamic override')

    def test_failure_in_dynamic_resolving_of_doc(self):
        lib = TestLibrary('dynlibs.FailingDynamicDocLib')
        assert_raises(DataError, getattr, lib, 'doc')

    def _assert_intro_doc(self, library_name, expected_doc):
        assert_equal(TestLibrary(library_name).doc, expected_doc)


class TestDynamicLibraryInitDocumentation(unittest.TestCase):

    def test_doc_from_class_init(self):
        self._assert_init_doc('dynlibs.StaticDocsLib', 'Init doc.')

    def test_doc_from_dynamic_method(self):
        self._assert_init_doc('dynlibs.DynamicDocsLib', 'Dynamic init doc.')

    def test_dynamic_doc_overrides_method_doc(self):
        self._assert_init_doc('dynlibs.StaticAndDynamicDocsLib',
                              'dynamic override')

    def test_failure_in_dynamic_resolving_of_doc(self):
        init = TestLibrary('dynlibs.FailingDynamicDocLib').init
        assert_raises(DataError, getattr, init, 'doc')

    def _assert_init_doc(self, library_name, expected_doc):
        assert_equal(TestLibrary(library_name).init.doc, expected_doc)


class TestSourceAndLineno(unittest.TestCase):

    def test_class(self):
        lib = TestLibrary('classes.NameLibrary')
        self._verify(lib, classes_source, 10)

    def test_class_in_package(self):
        from robot.variables.variables import __file__ as source
        lib = TestLibrary('robot.variables.Variables')
        self._verify(lib, source, 24)

    def test_dynamic(self):
        lib = TestLibrary('classes.ArgDocDynamicLibrary')
        self._verify(lib, classes_source, 215)

    def test_module(self):
        from module_library import __file__ as source
        lib = TestLibrary('module_library')
        self._verify(lib, source, 1)

    def test_package(self):
        from robot.variables import __file__ as source
        lib = TestLibrary('robot.variables')
        self._verify(lib, source, 1)

    def test_decorated(self):
        lib = TestLibrary('classes.Decorated')
        self._verify(lib, classes_source, 317)

    def test_no_class_statement(self):
        lib = TestLibrary('classes.NoClassDefinition')
        self._verify(lib, classes_source, -1)

    def _verify(self, lib, source, lineno):
        if source:
            source = re.sub(r'(\.pyc|\$py\.class)$', '.py', source)
            source = os.path.normpath(source)
        assert_equal(lib.source, source)
        assert_equal(lib.lineno, lineno)


class _FakeNamespace:
    def __init__(self):
        self.variables = _FakeVariableScope()
        self.uk_handlers = []
        self.test = None


class _FakeVariableScope:
    def __init__(self):
        self.variables = {}
    def replace_scalar(self, variable):
        return variable
    def replace_list(self, args, replace_until=None):
        return []
    def replace_string(self, variable):
        try:
            number = variable.replace('$', '').replace('{', '').replace('}', '')
            return int(number)
        except ValueError:
            pass
        try:
            return self.variables[variable]
        except KeyError:
            raise DataError("Non-existing variable '%s'" % variable)
    def __setitem__(self, key, value):
        self.variables.__setitem__(key, value)
    def __getitem__(self, key):
        return self.variables.get(key)


class _FakeOutput:
    def trace(self, str, write_if_flat=True):
        pass
    def log_output(self, output):
        pass


class _FakeAsynchronous:
    def is_loop_required(self, obj):
        return False


class _FakeContext:
    def __init__(self):
        self.output = _FakeOutput()
        self.namespace = _FakeNamespace()
        self.dry_run = False
        self.in_teardown = False
        self.variables = _FakeVariableScope()
        self.timeouts = set()
        self.test = None
        self.asynchronous = _FakeAsynchronous()


if __name__ == '__main__':
    unittest.main()



import unittest

from robot.reporting.logreportwriters import LogWriter
from robot.utils.asserts import assert_true, assert_equal


class LogWriterWithMockedWriting(LogWriter):

    def __init__(self, model):
        LogWriter.__init__(self, model)
        self.split_write_calls = []
        self.write_called = False

    def _write_split_log(self, index, keywords, strings, path):
        self.split_write_calls.append((index, keywords, strings, path))

    def _write_file(self, output, config, template):
        self.write_called = True



class TestLogWriter(unittest.TestCase):

    def test_splitting_log(self):
        class model:
            split_results = [((0, 1, 2, -1), ('*', '*1', '*2')),
                             ((0, 1, 0, 42), ('*','*x')),
                             (((1, 2), (3, 4, ())), ('*',))]
        writer = LogWriterWithMockedWriting(model)
        writer.write('mylog.html', None)
        assert_true(writer.write_called)
        assert_equal([(1, (0, 1, 2, -1), ('*', '*1', '*2'), 'mylog-1.js'),
                       (2, (0, 1, 0, 42), ('*', '*x'), 'mylog-2.js'),
                       (3, ((1, 2), (3, 4, ())), ('*',), 'mylog-3.js')],
                      writer.split_write_calls)


if __name__ == '__main__':
    unittest.main()



from robot.libraries.BuiltIn import BuiltIn
from robot.api import logger


class Importing:

    def __init__(self):
        BuiltIn().import_library('String')

    def kw_from_lib_with_importing_init(self):
        print('Keyword from library with importing __init__.')


class Initting:

    def __init__(self):
        # This initializes the accesses library.
        self.lib = BuiltIn().get_library_instance('InitImportingAndIniting.Initted')

    def kw_from_lib_with_initting_init(self):
        logger.info('Keyword from library with initting __init__.')
        self.lib.kw_from_lib_initted_by_init()


class Initted:

    def __init__(self, id):
        self.id = id

    def kw_from_lib_initted_by_init(self):
        print('Keyword from library initted by __init__ (id: %s).' % self.id)



from functools import wraps


__all__ = ['keyword_using_decorator', 'keyword_using_decorator_with_wraps']


def decorator(func):
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    return wrapper


def decorator_with_wraps(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    return wrapper


@decorator
def keyword_using_decorator(args, are_not, preserved=True):
    return '%s %s %s' % (args, are_not, preserved)


@decorator_with_wraps
def keyword_using_decorator_with_wraps(args, are, preserved=True):
    pass



import time
import datetime

def get_timestamp_from_date(*args):
    return int(time.mktime(datetime.datetime(*(int(arg) for arg in args)).timetuple()))

def get_current_time_zone():
    return time.altzone if time.localtime().tm_isdst else time.timezone



import time


def busy_sleep(seconds):
    max_time = time.time() + int(seconds)
    while time.time() < max_time:
        time.sleep(0)


def swallow_exception(timeout=3):
    try:
        busy_sleep(timeout)
    except:
        pass
    else:
        raise AssertionError('Expected exception did not occur!')



import sys

from remoteserver import RemoteServer, keyword


class KeywordTags:

    def no_tags(self):
        pass

    def doc_contains_tags_only(self):
        """Tags: foo, bar"""

    def doc_contains_tags_after_doc(self):
        """This is by doc.

        My doc has multiple lines.

        Tags: these, are, my, tags
        """

    @keyword
    def empty_robot_tags_means_no_tags(self):
        pass

    @keyword(tags=['foo', 'bar', 'FOO', '42'])
    def robot_tags(self):
        pass

    @keyword(tags=['foo', 'bar'])
    def robot_tags_and_doc_tags(self):
        """Tags: bar, zap"""


if __name__ == '__main__':
    RemoteServer(KeywordTags(), *sys.argv[1:])



scalar_from_cli_varfile = 'Scalar from variable file from cli'
scalar_from_cli_varfile_with_escapes = '1 \\ 2\\\\ ${inv}'
list_var_from_cli_varfile = 'Scalar list from variable file from cli'.split()
LIST__list_var_from_cli_varfile = 'List from variable file from cli'.split()
clivar = 'This value is not taken into use because var is overridden from cli'


from __future__ import annotations
from collections.abc import Mapping
from numbers import Integral
from typing import List


def concrete_types(a: int, b: bool, c: list):
    assert a == 42, repr(a)
    assert b is False, repr(b)
    assert c == [1, 'kaksi'], repr(c)


def abcs(a: Integral, b: Mapping):
    assert a == 42, repr(a)
    assert b == {'key': 'value'}, repr(b)


def typing_(a: List, b: List[int]):
    assert a == ['foo', 'bar'], repr(a)
    assert b == [1, 2, 3], repr(b)


# These cause exception with `typing.get_type_hints`
def invalid1(a: foo):
    assert a == 'xxx'


def invalid2(a: 1/0):
    assert a == 'xxx'



import unittest
import time
import glob
import sys
import threading
import tempfile
import signal
import logging
from io import StringIO
from os.path import abspath, curdir, dirname, exists, join
from os import chdir, getenv

from robot import run, run_cli, rebot, rebot_cli
from robot.model import SuiteVisitor
from robot.running import namespace
from robot.utils.asserts import assert_equal, assert_raises, assert_true

from resources.runningtestcase import RunningTestCase
from resources.Listener import Listener


ROOT = dirname(dirname(dirname(abspath(__file__))))
TEMP = getenv('TEMPDIR', tempfile.gettempdir())
OUTPUT_PATH = join(TEMP, 'output.xml')
REPORT_PATH = join(TEMP, 'report.html')
LOG_PATH = join(TEMP, 'log.html')
LOG = 'Log:     %s' % LOG_PATH


def run_without_outputs(*args, **kwargs):
    options = {'output': 'NONE', 'log': 'NoNe', 'report': None}
    options.update(kwargs)
    return run(*args, **options)


def assert_signal_handler_equal(signum, expected):
    sig = signal.getsignal(signum)
    assert_equal(sig, expected)


class StreamWithOnlyWriteAndFlush:

    def __init__(self):
        self._buffer = []

    def write(self, msg):
        self._buffer.append(msg)

    def flush(self):
        pass

    def getvalue(self):
        return ''.join(self._buffer)


class TestRun(RunningTestCase):
    data = join(ROOT, 'atest', 'testdata', 'misc', 'pass_and_fail.robot')
    warn = join(ROOT, 'atest', 'testdata', 'misc', 'warnings_and_errors.robot')
    nonex = join(TEMP, 'non-existing-file-this-is.robot')
    remove_files = [LOG_PATH, REPORT_PATH, OUTPUT_PATH]

    def test_run_once(self):
        assert_equal(run(self.data, outputdir=TEMP, report='none'), 1)
        self._assert_outputs([('Pass And Fail', 2), (LOG, 1), ('Report:', 0)])
        assert exists(LOG_PATH)

    def test_run_multiple_times(self):
        assert_equal(run_without_outputs(self.data), 1)
        assert_equal(run_without_outputs(self.data, name='New Name'), 1)
        self._assert_outputs([('Pass And Fail', 2), ('New Name', 2), (LOG, 0)])

    def test_run_fail(self):
        assert_equal(run(self.data, outputdir=TEMP), 1)
        self._assert_outputs(stdout=[('Pass And Fail', 2), (LOG, 1)])

    def test_run_error(self):
        assert_equal(run(self.nonex), 252)
        self._assert_outputs(stderr=[('[ ERROR ]', 1), (self.nonex, 1),
                                     ('--help', 1)])

    def test_custom_stdout(self):
        stdout = StringIO()
        assert_equal(run_without_outputs(self.data, stdout=stdout), 1)
        self._assert_output(stdout, [('Pass And Fail', 2), ('Output:', 1),
                                     ('Log:', 0), ('Report:', 0)])
        self._assert_outputs()

    def test_custom_stderr(self):
        stderr = StringIO()
        assert_equal(run_without_outputs(self.warn, stderr=stderr), 0)
        self._assert_output(stderr, [('[ WARN ]', 4), ('[ ERROR ]', 2)])
        self._assert_outputs([('Warnings And Errors', 2), ('Output:', 1),
                              ('Log:', 0), ('Report:', 0)])

    def test_custom_stdout_and_stderr_with_minimal_implementation(self):
        output = StreamWithOnlyWriteAndFlush()
        assert_equal(run_without_outputs(self.warn, stdout=output, stderr=output), 0)
        self._assert_output(output, [('[ WARN ]', 4), ('[ ERROR ]', 2),
                                     ('Warnings And Errors', 3), ('Output:', 1),
                                     ('Log:', 0), ('Report:', 0)])
        self._assert_outputs()

    def test_multi_options_as_single_string(self):
        assert_equal(run_without_outputs(self.data, include='?a??', skip='pass',
                                         skiponfailure='fail'), 0)
        self._assert_outputs([('2 tests, 0 passed, 0 failed, 2 skipped', 1)])

    def test_multi_options_as_tuples(self):
        assert_equal(run_without_outputs(self.data, exclude=('fail',), skip=('pass',),
                                         skiponfailure=('xxx', 'yyy')), 0)
        self._assert_outputs([('FAIL', 0)])
        self._assert_outputs([('1 test, 0 passed, 0 failed, 1 skipped', 1)])

    def test_listener_gets_notification_about_log_report_and_output(self):
        listener = join(ROOT, 'utest', 'resources', 'Listener.py')
        assert_equal(run(self.data, output=OUTPUT_PATH, report=REPORT_PATH,
                         log=LOG_PATH, listener=listener), 1)
        self._assert_outputs(stdout=[('[output {0}]'.format(OUTPUT_PATH), 1),
                                     ('[report {0}]'.format(REPORT_PATH), 1),
                                     ('[log {0}]'.format(LOG_PATH), 1),
                                     ('[listener close]', 1)])

    def test_pass_listener_as_instance(self):
        assert_equal(run_without_outputs(self.data, listener=Listener(1)), 1)
        self._assert_outputs([("[from listener 1]", 1)])

    def test_pass_listener_as_string(self):
        module_file = join(ROOT, 'utest', 'resources', 'Listener.py')
        assert_equal(run_without_outputs(self.data, listener=module_file+":1"), 1)
        self._assert_outputs([("[from listener 1]", 1)])

    def test_pass_listener_as_list(self):
        module_file = join(ROOT, 'utest', 'resources', 'Listener.py')
        assert_equal(run_without_outputs(self.data, listener=[module_file+":1", Listener(2)]), 1)
        self._assert_outputs([("[from listener 1]", 1), ("[from listener 2]", 1)])

    def test_pre_run_modifier_as_instance(self):
        class Modifier(SuiteVisitor):
            def start_suite(self, suite):
                suite.tests = [t for t in suite.tests if t.tags.match('pass')]
        assert_equal(run_without_outputs(self.data, prerunmodifier=Modifier()), 0)
        self._assert_outputs([('Pass       ', 1), ('Fail :: FAIL', 0)])

    def test_pre_rebot_modifier_as_instance(self):
        class Modifier(SuiteVisitor):
            def __init__(self):
                self.tests = []
            def visit_test(self, test):
                self.tests.append(test.name)
        modifier = Modifier()
        assert_equal(run(self.data, outputdir=TEMP, log=LOG_PATH, prerebotmodifier=modifier), 1)
        assert_equal(modifier.tests, ['Pass', 'Fail'])
        self._assert_outputs([('Pass       ', 1), ('Fail :: FAIL', 1)])

    def test_invalid_modifier(self):
        assert_equal(run_without_outputs(self.data, prerunmodifier=42), 1)
        self._assert_outputs([('Pass       ', 1), ('Fail :: FAIL', 1)],
                             [("[ ERROR ] Executing model modifier 'integer' "
                               "failed: AttributeError: ", 1)])

    def test_invalid_option_value(self):
        stderr = StringIO()
        assert_equal(run(self.data, loglevel='INV', stderr=stderr), 252)
        self._assert_output(stderr, [("[ ERROR ] Invalid value for option '--loglevel': "
                                      "Invalid level 'INV'.", 1)])
        self._assert_outputs()

    def test_invalid_option(self):
        stderr = StringIO()
        assert_equal(run(self.data, invalid=True, stderr=stderr), 252)
        self._assert_output(stderr, [("[ ERROR ] Invalid option 'invalid'.", 1)])
        self._assert_outputs()

    def test_run_cli_system_exits_by_default(self):
        exit = assert_raises(SystemExit, run_cli, ['-d', TEMP, self.data])
        assert_equal(exit.code, 1)

    def test_run_cli_optionally_returns_rc(self):
        rc = run_cli(['-d', TEMP, self.data], exit=False)
        assert_equal(rc, 1)


class TestRebot(RunningTestCase):
    data = join(ROOT, 'atest', 'testdata', 'rebot', 'created_normal.xml')
    nonex = join(TEMP, 'non-existing-file-this-is.xml')
    remove_files = [LOG_PATH, REPORT_PATH]

    def test_run_once(self):
        assert_equal(rebot(self.data, outputdir=TEMP, report='NONE'), 1)
        self._assert_outputs([(LOG, 1), ('Report:', 0)])
        assert exists(LOG_PATH)

    def test_run_multiple_times(self):
        assert_equal(rebot(self.data, outputdir=TEMP), 1)
        assert_equal(rebot(self.data, outputdir=TEMP, name='New Name'), 1)
        self._assert_outputs([(LOG, 2)])

    def test_run_fails(self):
        assert_equal(rebot(self.nonex), 252)
        assert_equal(rebot(self.data, outputdir=TEMP), 1)
        self._assert_outputs(stdout=[(LOG, 1)],
                             stderr=[('[ ERROR ]', 1), (self.nonex, (1, 2)),
                                     ('--help', 1)])

    def test_custom_stdout(self):
        stdout = StringIO()
        assert_equal(rebot(self.data, report='None', stdout=stdout,
                           outputdir=TEMP), 1)
        self._assert_output(stdout, [('Log:', 1), ('Report:', 0)])
        self._assert_outputs()

    def test_custom_stdout_and_stderr_with_minimal_implementation(self):
        output = StreamWithOnlyWriteAndFlush()
        assert_equal(rebot(self.data, log='NONE', report='NONE', stdout=output,
                           stderr=output), 252)
        assert_equal(rebot(self.data, report='NONE', stdout=output,
                           stderr=output, outputdir=TEMP), 1)
        self._assert_output(output, [('[ ERROR ] No outputs created', 1),
                                     ('--help', 1), ('Log:', 1), ('Report:', 0)])
        self._assert_outputs()

    def test_pre_rebot_modifier_as_instance(self):
        class Modifier(SuiteVisitor):
            def __init__(self):
                self.tests = []
            def visit_test(self, test):
                self.tests.append(test.name)
                test.status = 'FAIL'
        modifier = Modifier()
        assert_equal(rebot(self.data, outputdir=TEMP,
                           prerebotmodifier=modifier), 3)
        assert_equal(modifier.tests, ['Test 1.1', 'Test 1.2', 'Test 2.1'])

    def test_invalid_option_value(self):
        stderr = StringIO()
        assert_equal(rebot(self.data, loglevel='INFO:INV', stderr=stderr), 252)
        self._assert_output(stderr, [("[ ERROR ] Invalid value for option '--loglevel': "
                                      "Invalid level 'INV'.", 1)])
        self._assert_outputs()

    def test_invalid_option(self):
        stderr = StringIO()
        assert_equal(rebot(self.data, inv=True, stderr=stderr, bad=-1), 252)
        self._assert_output(stderr, [("[ ERROR ] Invalid options 'inv' and 'bad'.", 1)])
        self._assert_outputs()

    def test_rebot_cli_system_exits_by_default(self):
        exit = assert_raises(SystemExit, rebot_cli, ['-d', TEMP, self.data])
        assert_equal(exit.code, 1)

    def test_rebot_cli_optionally_returns_rc(self):
        rc = rebot_cli(['-d', TEMP, self.data], exit=False)
        assert_equal(rc, 1)


class TestStateBetweenTestRuns(RunningTestCase):
    data = join(ROOT, 'atest', 'testdata', 'misc', 'normal.robot')

    def test_importer_caches_are_cleared_between_runs(self):
        self._run(self.data)
        lib = self._import_library()
        res = self._import_resource()
        self._run(self.data)
        assert_true(lib is not self._import_library())
        assert_true(res is not self._import_resource())

    def _run(self, data, rc=None, **config):
        self._clear_outputs()
        returned_rc = run_without_outputs(data, outputdir=TEMP, **config)
        if rc is not None:
            assert_equal(returned_rc, rc)

    def _import_library(self):
        return namespace.IMPORTER.import_library('BuiltIn', None, None, None)

    def _import_resource(self):
        resource = join(ROOT, 'atest', 'testdata', 'core', 'resources.robot')
        return namespace.IMPORTER.import_resource(resource)

    def test_clear_namespace_between_runs(self):
        data = join(ROOT, 'atest', 'testdata', 'variables', 'commandline_variables.robot')
        self._run(data, test=['NormalText'], variable=['NormalText:Hello'], rc=0)
        self._run(data, test=['NormalText'], rc=1)

    def test_reset_logging_conf(self):
        assert_equal(logging.getLogger().handlers, [])
        assert_equal(logging.raiseExceptions, 1)
        self._run(join(ROOT, 'atest', 'testdata', 'misc', 'normal.robot'))
        assert_equal(logging.getLogger().handlers, [])
        assert_equal(logging.raiseExceptions, 1)

    def test_listener_unregistration(self):
        listener = join(ROOT, 'utest', 'resources', 'Listener.py')
        self._run(self.data, listener=listener+':1', rc=0)
        self._assert_outputs([("[from listener 1]", 1), ("[listener close]", 1)])
        self._run(self.data, rc=0)
        self._assert_outputs([("[from listener 1]", 0), ("[listener close]", 0)])

    def test_rerunfailed_is_not_persistent(self):
        # https://github.com/robotframework/robotframework/issues/2437
        data = join(ROOT, 'atest', 'testdata', 'misc', 'pass_and_fail.robot')
        self._run(data, output=OUTPUT_PATH, rc=1)
        self._run(data, rerunfailed=OUTPUT_PATH, rc=1)
        self._run(self.data, output=OUTPUT_PATH, rc=0)
        assert_equal(rebot(OUTPUT_PATH, log=LOG_PATH, report=None), 0)


class TestTimestampOutputs(RunningTestCase):
    output = join(TEMP, 'output-ts-*.xml')
    report = join(TEMP, 'report-ts-*.html')
    log = join(TEMP, 'log-ts-*.html')
    remove_files = [output, report, log]

    def test_different_timestamps_when_run_multiple_times(self):
        self.run_tests()
        output1, = self.find_results(self.output, 1)
        report1, = self.find_results(self.report, 1)
        log1, = self.find_results(self.log, 1)
        self.wait_until_next_second()
        self.run_tests()
        output21, output22 = self.find_results(self.output, 2)
        report21, report22 = self.find_results(self.report, 2)
        log21, log22 = self.find_results(self.log, 2)
        assert_equal(output1, output21)
        assert_equal(report1, report21)
        assert_equal(log1, log21)

    def run_tests(self):
        data = join(ROOT, 'atest', 'testdata', 'misc', 'pass_and_fail.robot')
        assert_equal(run(data, timestampoutputs=True, outputdir=TEMP,
                         output='output-ts.xml', report='report-ts.html',
                         log='log-ts'), 1)

    def find_results(self, pattern, expected):
        matches = glob.glob(pattern)
        assert_equal(len(matches), expected)
        return sorted(matches)

    def wait_until_next_second(self):
        start = time.localtime()[5]
        while time.localtime()[5] == start:
            time.sleep(0.01)


class TestSignalHandlers(unittest.TestCase):
    data = join(ROOT, 'atest', 'testdata', 'misc', 'pass_and_fail.robot')

    def test_original_signal_handlers_are_restored(self):
        orig_sigint = signal.getsignal(signal.SIGINT)
        orig_sigterm = signal.getsignal(signal.SIGTERM)
        my_sigterm = lambda signum, frame: None
        signal.signal(signal.SIGTERM, my_sigterm)
        try:
            run_without_outputs(self.data, stdout=StringIO())
            assert_signal_handler_equal(signal.SIGINT, orig_sigint)
            assert_signal_handler_equal(signal.SIGTERM, my_sigterm)
        finally:
            signal.signal(signal.SIGINT, orig_sigint)
            signal.signal(signal.SIGTERM, orig_sigterm)

    def test_dont_register_signal_handlers_when_run_on_thread(self):
        stream = StringIO()
        thread = threading.Thread(target=run_without_outputs, args=(self.data,),
                                  kwargs=dict(stdout=stream, stderr=stream))
        thread.start()
        thread.join()
        output = stream.getvalue()
        assert_true('ERROR' not in output.upper(), 'Errors:\n%s' % output)


class TestRelativeImportsFromPythonpath(RunningTestCase):
    data = join(abspath(dirname(__file__)), 'import_test.robot')

    def setUp(self):
        self._orig_path = abspath(curdir)
        chdir(ROOT)
        sys.path.append(join('atest', 'testresources'))

    def tearDown(self):
        chdir(self._orig_path)
        sys.path.pop()

    def test_importing_library_from_pythonpath(self):
        errors = StringIO()
        run(self.data, outputdir=TEMP, stdout=StringIO(), stderr=errors)
        self._assert_output(errors, '')


if __name__ == '__main__':
    unittest.main()



from subprocess import call

def test_env_var_in_child_process(var):
    rc = call(['python', '-c', 'import os, sys; sys.exit("%s" in os.environ)' % var])
    if rc !=1 :
        raise AssertionError("Variable '%s' did not exist in child environment" % var)




import unittest
import warnings

from robot.model import TestSuite, TestCase, Keyword, Keywords
from robot.utils.asserts import (assert_equal, assert_not_equal, assert_true,
                                 assert_raises)


class TestKeyword(unittest.TestCase):

    def test_id_without_parent(self):
        assert_equal(Keyword().id, 'k1')
        assert_equal(Keyword(type=Keyword.SETUP).id, 'k1')
        assert_equal(Keyword(type=Keyword.TEARDOWN).id, 'k1')

    def test_suite_setup_and_teardown_id(self):
        suite = TestSuite()
        assert_equal(suite.setup.id, None)
        assert_equal(suite.teardown.id, None)
        suite.teardown.config(name='T')
        assert_equal(suite.teardown.id, 's1-k1')
        suite.setup.config(name='S')
        assert_equal(suite.setup.id, 's1-k1')
        assert_equal(suite.teardown.id, 's1-k2')

    def test_test_setup_and_teardown_id(self):
        test = TestSuite().tests.create()
        assert_equal(test.setup.id, None)
        assert_equal(test.teardown.id, None)
        test.setup.config(name='S')
        test.teardown.config(name='T')
        assert_equal(test.setup.id, 's1-t1-k1')
        assert_equal(test.teardown.id, 's1-t1-k2')
        test.body.create_keyword()
        assert_equal(test.setup.id, 's1-t1-k1')
        assert_equal(test.teardown.id, 's1-t1-k3')

    def test_test_body_id(self):
        kws = [Keyword(), Keyword(), Keyword()]
        TestSuite().tests.create().body.extend(kws)
        assert_equal([k.id for k in kws], ['s1-t1-k1', 's1-t1-k2', 's1-t1-k3'])

    def test_id_with_for_parent(self):
        for_body = TestCase().body.create_for().body
        assert_equal(for_body.create_keyword().id, 't1-k1-k1')
        assert_equal(for_body.create_keyword().id, 't1-k1-k2')

    def test_id_with_if_parent(self):
        if_body = TestCase().body.create_if().body
        assert_equal(if_body.create_branch().id, 't1-k1')
        assert_equal(if_body.create_branch().body.create_keyword().id, 't1-k2-k1')
        assert_equal(if_body.create_branch().body.create_keyword().id, 't1-k3-k1')

    def test_id_with_messages_in_body(self):
        from robot.result.model import Keyword
        kw = Keyword()
        assert_equal(kw.body.create_message().id, 'k1-m1')
        assert_equal(kw.body.create_keyword().id, 'k1-k1')
        assert_equal(kw.body.create_message().id, 'k1-m2')
        assert_equal(kw.body.create_keyword().id, 'k1-k2')

    def test_string_reprs(self):
        for kw, exp_str, exp_repr in [
            (Keyword(),
             '',
             "Keyword(name='', args=(), assign=())"),
            (Keyword('name'),
             'name',
             "Keyword(name='name', args=(), assign=())"),
            (Keyword(None),
             'None',
             "Keyword(name=None, args=(), assign=())"),
            (Keyword('Name', args=('a1', 'a2')),
             'Name    a1    a2',
             "Keyword(name='Name', args=('a1', 'a2'), assign=())"),
            (Keyword('Name', assign=('${x}', '${y}')),
             '${x}    ${y}    Name',
             "Keyword(name='Name', args=(), assign=('${x}', '${y}'))"),
            (Keyword('Name', assign=['${x}='], args=['x']),
             '${x}=    Name    x',
             "Keyword(name='Name', args=('x',), assign=('${x}=',))"),
            (Keyword('Name', args=(1, 2, 3)),
             'Name    1    2    3',
             "Keyword(name='Name', args=(1, 2, 3), assign=())"),
            (Keyword(assign=['${ã}'], name='ä', args=['å']),
             '${ã}    ä    å',
             "Keyword(name='ä', args=('å',), assign=('${ã}',))")
        ]:
            assert_equal(str(kw), exp_str)
            assert_equal(repr(kw), 'robot.model.' + exp_repr)

    def test_slots(self):
        assert_raises(AttributeError, setattr, Keyword(), 'attr', 'value')

    def test_copy(self):
        kw = Keyword(name='Keyword', args=['args'])
        copy = kw.copy()
        assert_equal(kw.name, copy.name)
        copy.name += ' copy'
        assert_not_equal(kw.name, copy.name)
        assert_equal(kw.args, copy.args)

    def test_copy_with_attributes(self):
        kw = Keyword(name='Orig', args=('orig',))
        copy = kw.copy(name='New', args=['new'])
        assert_equal(copy.name, 'New')
        assert_equal(copy.args, ('new',))

    def test_deepcopy(self):
        kw = Keyword(name='Keyword', args=['a'])
        copy = kw.deepcopy()
        assert_equal(kw.name, copy.name)
        assert_equal(kw.args, copy.args)

    def test_deepcopy_with_attributes(self):
        copy = Keyword(name='Orig').deepcopy(name='New', args=['New'])
        assert_equal(copy.name, 'New')
        assert_equal(copy.args, ('New',))

    def test_copy_and_deepcopy_with_non_existing_attributes(self):
        assert_raises(AttributeError, Keyword().copy, bad='attr')
        assert_raises(AttributeError, Keyword().deepcopy, bad='attr')


class TestKeywords(unittest.TestCase):

    def test_deprecation(self):
        with warnings.catch_warnings(record=True) as w:
            Keywords()
            assert_true('deprecated' in str(w[0].message))


if __name__ == '__main__':
    unittest.main()



import os
import tempfile


TEMPDIR = os.getenv('TEMPDIR', tempfile.gettempdir())


class LinenoAndSource:
    ROBOT_LISTENER_API_VERSION = 2

    def __init__(self):
        self.suite_output = open(os.path.join(TEMPDIR, 'LinenoAndSourceSuite.txt'), 'w')
        self.test_output = open(os.path.join(TEMPDIR, 'LinenoAndSourceTests.txt'), 'w')
        self.output = None

    def start_suite(self, name, attrs):
        self.output = self.suite_output
        self.report('START', type='SUITE', name=name, **attrs)

    def end_suite(self, name, attrs):
        self.output = self.suite_output
        self.report('END',  type='SUITE', name=name, **attrs)

    def start_test(self, name, attrs):
        self.output = self.test_output
        self.report('START', type='TEST', name=name, **attrs)
        self.output = open(os.path.join(TEMPDIR, name + '.txt'), 'w')

    def end_test(self, name, attrs):
        self.output.close()
        self.output = self.test_output
        self.report('END', type='TEST', name=name, **attrs)
        self.output = self.suite_output

    def start_keyword(self, name, attrs):
        self.report('START', **attrs)

    def end_keyword(self, name, attrs):
        self.report('END', **attrs)

    def close(self):
        self.suite_output.close()
        self.test_output.close()

    def report(self, event, type, source, lineno=-1, name=None, kwname=None,
               status=None, **ignore):
        info = [event, type, name or kwname, lineno, source]
        if status:
            info.append(status)
        self.output.write('\t'.join(str(i) for i in info) + '\n')



import sys

from remoteserver import RemoteServer, keyword


class BulkLoadRemoteServer(RemoteServer):

    def _register_functions(self):
        """
        Individual get_keyword_* methods are not registered.
        This removes the fall back scenario should get_library_information fail.
        """
        self.register_function(self.get_library_information)
        self.register_function(self.run_keyword)

    def get_library_information(self):
        info_dict = {'__init__': {'doc': '__init__ documentation.'},
                     '__intro__': {'doc': '__intro__ documentation.'}}
        for kw in self.get_keyword_names():
            info_dict[kw] = dict(
                args=['arg', '*extra'] if kw == 'some_keyword' else ['arg=None'],
                doc="Documentation for '%s'." % kw,
                tags=['tag'],
                types=['bool'] if kw == 'some_keyword' else ['int']
            )
        return info_dict


class The10001KeywordsLibrary:

    def __init__(self):
        for i in range(10000):
            setattr(self, 'keyword_%d' % i, lambda result=str(i): result)

    def some_keyword(self, arg, *extra):
        return 'yes' if arg else 'no'


if __name__ == '__main__':
    BulkLoadRemoteServer(The10001KeywordsLibrary(), *sys.argv[1:])



import codecs
import os
import tempfile
import unittest
from io import BytesIO, StringIO
from pathlib import Path

from robot.utils import FileReader
from robot.utils.asserts import assert_equal, assert_raises


TEMPDIR = os.getenv('TEMPDIR') or tempfile.gettempdir()
PATH = os.path.join(TEMPDIR, 'filereader.test')
STRING = u'Hyv\xe4\xe4\nty\xf6t\xe4\nC\u043f\u0430\u0441\u0438\u0431\u043e\n'


def assert_reader(reader, name=PATH):
    assert_equal(reader.read(), STRING, formatter=repr)
    assert_equal(reader.name, name)
    assert_open(reader.file)


def assert_open(*files):
    for f in files:
        assert_equal(f.closed, False)


def assert_closed(*files):
    for f in files:
        assert_equal(f.closed, True)


class TestReadFile(unittest.TestCase):
    BOM = b''
    created_files = set()

    @classmethod
    def setUpClass(cls):
        cls._create()

    @classmethod
    def _create(cls, content=STRING, path=PATH, encoding='UTF-8'):
        with open(path, 'wb') as f:
            f.write(cls.BOM)
            f.write(content.replace('\n', os.linesep).encode(encoding))
        cls.created_files.add(path)

    @classmethod
    def tearDownClass(cls):
        for path in cls.created_files:
            os.remove(path)
        cls.created_files = set()

    def test_path_as_string(self):
        with FileReader(PATH) as reader:
            assert_reader(reader)
        assert_closed(reader.file)

    def test_open_text_file(self):
        with open(PATH, encoding='UTF-8') as f:
            with FileReader(f) as reader:
                assert_reader(reader)
            assert_open(f, reader.file)
        assert_closed(f, reader.file)

    def test_path_as_pathlib_path(self):
        with FileReader(Path(PATH)) as reader:
            assert_reader(reader)
        assert_closed(reader.file)

    def test_codecs_open_file(self):
        with codecs.open(PATH, encoding='UTF-8') as f:
            with FileReader(f) as reader:
                assert_reader(reader)
            assert_open(f, reader.file)
        assert_closed(f, reader.file)

    def test_open_binary_file(self):
        with open(PATH, 'rb') as f:
            with FileReader(f) as reader:
                assert_reader(reader)
            assert_open(f, reader.file)
        assert_closed(f, reader.file)

    def test_stringio(self):
        f = StringIO(STRING)
        with FileReader(f) as reader:
            assert_reader(reader, '<in-memory file>')
        assert_open(f)

    def test_bytesio(self):
        f = BytesIO(self.BOM + STRING.encode('UTF-8'))
        with FileReader(f) as reader:
            assert_reader(reader, '<in-memory file>')
        assert_open(f)

    def test_text(self):
        with FileReader(STRING, accept_text=True) as reader:
            assert_reader(reader, '<in-memory file>')
        assert_closed(reader.file)

    def test_text_with_special_chars(self):
        for text in '!"#¤%&/()=?', '*** Test Cases ***', 'in:va:lid':
            with FileReader(text, accept_text=True) as reader:
                assert_equal(reader.read(), text)

    def test_text_when_text_is_not_accepted(self):
        assert_raises(IOError, FileReader, STRING)

    def test_readlines(self):
        with FileReader(PATH) as reader:
            assert_equal(list(reader.readlines()), STRING.splitlines(True))

    def test_invalid_encoding(self):
        russian = STRING.split()[-1]
        path = os.path.join(TEMPDIR, 'filereader.iso88595')
        self._create(russian, path, encoding='ISO-8859-5')
        with FileReader(path) as reader:
            assert_raises(UnicodeDecodeError, reader.read)


class TestReadFileWithBom(TestReadFile):
    BOM = codecs.BOM_UTF8


if __name__ == '__main__':
    unittest.main()



class _BaseDynamicLibrary:

    def get_keyword_names(self):
        return []

    def run_keyword(self, name, *args):
        return None

class StaticDocsLib(_BaseDynamicLibrary):
    """This is lib intro."""
    def __init__(self, some=None, args=[]):
        """Init doc."""

class DynamicDocsLib(_BaseDynamicLibrary):
    def __init__(self, *args): pass

    def get_keyword_documentation(self, name):
        if name == '__intro__':
            return 'Dynamic intro doc.'
        if name == '__init__':
            return 'Dynamic init doc.'
        return ''

class StaticAndDynamicDocsLib(_BaseDynamicLibrary):
    """This is static doc."""
    def __init__(self, an_arg=None):
        """This is static doc."""
    def get_keyword_documentation(self, name):
        if name == '__intro__':
            return 'dynamic override'
        if name == '__init__':
            return 'dynamic override'
        return ''

class FailingDynamicDocLib(_BaseDynamicLibrary):
    """intro-o-o"""
    def __init__(self):
        """initoo-o-o"""
    def get_keyword_documentation(self, name):
        raise RuntimeError('Failing in get_keyword_documentation')




import json

from robot.api import logger


class WrongStat(AssertionError):
    ROBOT_CONTINUE_ON_FAILURE = True


def get_total_stats(path):
    return get_all_stats(path)[0]


def get_tag_stats(path):
    return get_all_stats(path)[1]


def get_suite_stats(path):
    return get_all_stats(path)[2]


def get_all_stats(path):
    stats = _get_output_line(path, 'window.output["stats"]')
    total, tags, suite = json.loads(stats)
    return total, tags, suite


def _get_output_line(path, prefix):
    logger.info("Getting '%s' from '<a href=\"file://%s\">%s</a>'."
                % (prefix, path, path), html=True)
    prefix += ' = '
    with open(path, encoding='UTF-8') as file:
        for line in file:
            if line.startswith(prefix):
                logger.info('Found: %s' % line)
                return line[len(prefix):-2]


def verify_stat(stat, *attrs):
    stat.pop('elapsed')
    expected = dict(_get_expected_stat(attrs))
    if stat != expected:
        raise WrongStat('\n%-9s: %s\n%-9s: %s' % ('Got', stat, 'Expected', expected))


def _get_expected_stat(attrs):
    for key, value in (a.split(':', 1) for a in attrs):
        value = int(value) if value.isdigit() else str(value)
        yield str(key), value


def get_expand_keywords(path):
    expand = _get_output_line(path, 'window.output["expand_keywords"]')
    return json.loads(expand)



from robot.libraries.BuiltIn import BuiltIn, register_run_keyword


class RegisteredClass:
    def run_keyword_if_method(self, expression, name, *args):
        return BuiltIn().run_keyword_if(expression, name, *args)

    def run_keyword_method(self, name, *args):
        return BuiltIn().run_keyword(name, *args)

register_run_keyword("RegisteredClass", "Run Keyword If Method", 2)
register_run_keyword("RegisteredClass", "run_keyword_method", 1)



class NewStyleClassLibrary:

    def mirror(self, arg):
        arg = list(arg)
        arg.reverse()
        return ''.join(arg)

    @property
    def property_getter(self):
        raise SystemExit('This should not be called, ever!!!')

    @property
    def _property_getter(self):
        raise SystemExit('This should not be called, ever!!!')


class NewStyleClassArgsLibrary:

    def __init__(self, param):
        self.get_param = lambda self: param


class MyMetaClass(type):

    def __new__(cls, name, bases, ns):
        ns['kw_created_by_metaclass'] = lambda self, arg: arg.upper()
        return type.__new__(cls, name, bases, ns)

    def method_in_metaclass(cls):
        pass


class MetaClassLibrary(metaclass=MyMetaClass):

    def greet(self, name):
        return 'Hello %s!' % name



import unittest

from robot.utils.asserts import (assert_equal, assert_false, assert_not_equal,
                                 assert_true, assert_raises)
from robot.utils import seq2str
from robot.model.tags import Tags, TagPattern, TagPatterns


class TestTags(unittest.TestCase):

    def test_empty_init(self):
        assert_equal(list(Tags()), [])

    def test_init_with_string(self):
        assert_equal(list(Tags('string')), ['string'])

    def test_init_with_iterable_and_normalization_and_sorting(self):
        for inp in [['T 1', 't2', 't_3'],
                    ('t2', 'T 1', 't_3'),
                    ('t2', 'T 1', 't_3') + ('t2', 'T 1', 't_3'),
                    ('t2', 'T 2', '__T__2__', 'T 1', 't1', 't_1', 't_3', 't3'),
                    ('', 'T 1', '', 't2', 't_3', 'NONE', 'None')]:
            assert_equal(list(Tags(inp)), ['T 1', 't2', 't_3'])

    def test_init_with_non_strings(self):
        assert_equal(list(Tags([2, True, None])), ['2', 'True'])

    def test_init_with_none(self):
        assert_equal(list(Tags(None)), [])

    def test_robot(self):
        assert_equal(Tags().robot('x'), False)
        assert_equal(Tags('robot:x').robot('x'), True)
        assert_equal(Tags(['ROBOT : X']).robot('x'), True)
        assert_equal(Tags('robot:x:y').robot('x:y'), True)
        assert_equal(Tags('robot:x').robot('y'), False)

    def test_add_string(self):
        tags = Tags(['Y'])
        tags.add('x')
        assert_equal(list(tags), ['x', 'Y'])

    def test_add_iterable(self):
        tags = Tags(['A'])
        tags.add(('b b', '', 'a', 'NONE'))
        tags.add(Tags(['BB', 'C']))
        assert_equal(list(tags), ['A', 'b b', 'C'])

    def test_remove_string(self):
        tags = Tags(['a', 'B B'])
        tags.remove('a')
        assert_equal(list(tags), ['B B'])
        tags.remove('bb')
        assert_equal(list(tags), [])

    def test_remove_non_existing(self):
        tags = Tags(['a'])
        tags.remove('nonex')
        assert_equal(list(tags), ['a'])

    def test_remove_iterable(self):
        tags = Tags(['a', 'B B'])
        tags.remove(['nonex', '', 'A'])
        tags.remove(Tags('__B_B__'))
        assert_equal(list(tags), [])

    def test_remove_using_pattern(self):
        tags = Tags(['t1', 't2', '1', '1more'])
        tags.remove('?2')
        assert_equal(list(tags), ['1', '1more', 't1'])
        tags.remove('*1*')
        assert_equal(list(tags), [])

    def test_add_and_remove_none(self):
        tags = Tags(['t'])
        tags.add(None)
        tags.remove(None)
        assert_equal(list(tags), ['t'])

    def test_contains(self):
        assert_true('a' in Tags(['a', 'b']))
        assert_true('c' not in Tags(['a', 'b']))
        assert_true('AA' in Tags(['a_a', 'b']))

    def test_contains_pattern(self):
        assert_true('a*' in Tags(['a', 'b']))
        assert_true('a*' in Tags(['u2', 'abba']))
        assert_true('a?' not in Tags(['a', 'abba']))

    def test_length(self):
        assert_equal(len(Tags()), 0)
        assert_equal(len(Tags(['a', 'b'])), 2)

    def test_truth(self):
        assert_true(not Tags())
        assert_true(not Tags('NONE'))
        assert_true(Tags(['a']))

    def test_str(self):
        assert_equal(str(Tags()), '[]')
        assert_equal(str(Tags(['y', "X'X", 'Y'])), "[X'X, y]")
        assert_equal(str(Tags(['\xe4', 'a'])), '[a, \xe4]')

    def test_repr(self):
        for tags in ([], ['y', "X'X"], ['\xe4', 'a']):
            assert_equal(repr(Tags(tags)), repr(sorted(tags)))

    def test__add__list(self):
        tags = Tags(['xx', 'yy'])
        new_tags = tags + ['zz', 'ee', 'XX']
        assert_true(isinstance(new_tags, Tags))
        assert_equal(list(tags), ['xx', 'yy'])
        assert_equal(list(new_tags), ['ee', 'xx', 'yy', 'zz'])

    def test__add__tags(self):
        tags1 = Tags(['xx', 'yy'])
        tags2 = Tags(['zz', 'ee', 'XX'])
        new_tags = tags1 + tags2
        assert_true(isinstance(new_tags, Tags))
        assert_equal(list(tags1), ['xx', 'yy'])
        assert_equal(list(tags2), ['ee', 'XX', 'zz'])
        assert_equal(list(new_tags), ['ee', 'xx', 'yy', 'zz'])

    def test__add__None(self):
        tags = Tags(['xx', 'yy'])
        new_tags = tags + None
        assert_true(isinstance(new_tags, Tags))
        assert_equal(list(tags), ['xx', 'yy'])
        assert_equal(list(new_tags), list(tags))
        assert_true(new_tags is not tags)

    def test_getitem_with_index(self):
        tags = Tags(['2', '0', '1'])
        assert_equal(tags[0], '0')
        assert_equal(tags[1], '1')
        assert_equal(tags[2], '2')

    def test_getitem_with_slice(self):
        tags = Tags(['2', '0', '1'])
        self._verify_slice(tags[:], ['0', '1', '2'])
        self._verify_slice(tags[1:], ['1', '2'])
        self._verify_slice(tags[1:-1], ['1'])
        self._verify_slice(tags[1:-2], [])
        self._verify_slice(tags[::2], ['0', '2'])

    def _verify_slice(self, sliced, expected):
        assert_true(isinstance(sliced, Tags))
        assert_equal(list(sliced), expected)

    def test__eq__(self):
        assert_equal(Tags(['x']), Tags(['x']))
        assert_equal(Tags(['X']), Tags(['x']))
        assert_equal(Tags(['X', 'YZ']), Tags(('x', 'y_z')))
        assert_not_equal(Tags(['X']), Tags(['Y']))

    def test__eq__converts_other_to_tags(self):
        assert_equal(Tags(['X']), ['x'])
        assert_equal(Tags(['X']), 'x')
        assert_not_equal(Tags(['X']), 'y')

    def test__eq__with_other_that_cannot_be_converted_to_tags(self):
        assert_not_equal(Tags(), 1)
        assert_not_equal(Tags(), None)

    def test__eq__normalized(self):
        assert_equal(Tags(['Hello world', 'Foo', 'Not_world']),
                     Tags(['nOT WORLD', 'FOO', 'hello world']))

    def test__slots__(self):
        assert_raises(AttributeError, setattr, Tags(), 'attribute', 'value')


class TestNormalizing(unittest.TestCase):

    def test_empty(self):
        self._verify([], [])

    def test_case_and_space(self):
        for inp in ['lower'], ['MiXeD', 'UPPER'], ['a few', 'spaces here']:
            self._verify(inp, inp)

    def test_underscore(self):
        self._verify(['a_tag', 'a tag', 'ATag'], ['a_tag'])
        self._verify(['tag', '_t_a_g_'], ['tag'])

    def test_remove_empty_and_none(self):
        for inp in ['', 'X', '', '  ', '\n'], ['none', 'N O N E', 'X', '', '_']:
            self._verify(inp, ['X'])

    def test_remove_dupes(self):
        for inp in ['dupe', 'DUPE', ' d u p e '], ['d U', 'du', 'DU', 'Du']:
            self._verify(inp, [inp[0]])

    def test_sorting(self):
        for inp, exp in [(['SORT', '1', 'B', '2', 'a'],
                          ['1', '2', 'a', 'B', 'SORT']),
                         (['all', 'A LL', 'NONE', '10', '1', 'A', 'a', '', 'b'],
                          ['1', '10', 'A', 'all', 'b'])]:
            self._verify(inp, exp)

    def _verify(self, tags, expected):
        assert_equal(list(Tags(tags)), expected)


class TestTagPatterns(unittest.TestCase):

    def test_single_pattern(self):
        patterns = TagPatterns(['x', 'y', 'z*'])
        assert_false(patterns.match([]))
        assert_false(patterns.match(['no', 'match']))
        assert_true(patterns.match(['x']))
        assert_true(patterns.match(['xxx', 'zzz']))

    def test_and(self):
        patterns = TagPatterns(['xANDy', '???ANDz'])
        assert_false(patterns.match([]))
        assert_false(patterns.match(['x']))
        assert_true(patterns.match(['x', 'y', 'z']))
        assert_true(patterns.match(['123', 'y', 'z']))

    def test_multiple_ands(self):
        patterns = TagPatterns(['xANDyANDz'])
        assert_false(patterns.match([]))
        assert_false(patterns.match(['x']))
        assert_false(patterns.match(['x', 'y']))
        assert_true(patterns.match(['x', 'Y', 'z']))
        assert_true(patterns.match(['a', 'y', 'z', 'b', 'X']))

    def test_or(self):
        patterns = TagPatterns(['xORy', '???ORz'])
        assert_false(patterns.match([]))
        assert_false(patterns.match(['a', 'b', '12', '1234']))
        assert_true(patterns.match(['x']))
        assert_true(patterns.match(['Y']))
        assert_true(patterns.match(['123']))
        assert_true(patterns.match(['Z']))
        assert_true(patterns.match(['x', 'y', 'z']))
        assert_true(patterns.match(['123', 'a', 'b', 'c', 'd']))
        assert_true(patterns.match(['a', 'b', 'c', 'd', 'Z']))

    def test_multiple_ors(self):
        patterns = TagPatterns(['xORyORz'])
        assert_false(patterns.match([]))
        assert_false(patterns.match(['xxx']))
        assert_true(all(patterns.match([c]) for c in 'XYZ'))
        assert_true(all(patterns.match(['a', 'b', c, 'd']) for c in 'xyz'))
        assert_true(patterns.match(['x', 'y']))
        assert_true(patterns.match(['x', 'Y', 'z']))

    def test_ands_and_ors(self):
        for pattern in AndOrPatternGenerator(max_length=5):
            expected = eval(pattern.lower())
            assert_equal(TagPattern.from_string(pattern).match('1'), expected)

    def test_not(self):
        patterns = TagPatterns(['xNOTy', '???NOT?'])
        assert_false(patterns.match([]))
        assert_false(patterns.match(['x', 'y']))
        assert_false(patterns.match(['123', 'y', 'z']))
        assert_true(patterns.match(['x']))
        assert_true(patterns.match(['123', 'xx']))

    def test_not_and_and(self):
        patterns = TagPatterns(['xNOTyANDz', 'aANDbNOTc',
                                '1 AND 2? AND 3?? NOT 4* AND 5* AND 6*'])
        assert_false(patterns.match([]))
        assert_false(patterns.match(['x', 'y', 'z']))
        assert_true(patterns.match(['x', 'y']))
        assert_true(patterns.match(['x']))
        assert_false(patterns.match(['a', 'b', 'c']))
        assert_false(patterns.match(['a']))
        assert_false(patterns.match(['b']))
        assert_true(patterns.match(['a', 'b']))
        assert_true(patterns.match(['a', 'b', 'xxxx']))
        assert_false(patterns.match(['1', '22', '33']))
        assert_false(patterns.match(['1', '22', '333', '4', '5', '6']))
        assert_true(patterns.match(['1', '22', '333']))
        assert_true(patterns.match(['1', '22', '333', '4', '5', '7']))

    def test_not_and_or(self):
        patterns = TagPatterns(['xNOTyORz', 'aORbNOTc',
                                '1 OR 2? OR 3?? NOT 4* OR 5* OR 6*'])
        assert_false(patterns.match([]))
        assert_false(patterns.match(['x', 'y', 'z']))
        assert_false(patterns.match(['x', 'y']))
        assert_false(patterns.match(['Z', 'x']))
        assert_true(patterns.match(['x']))
        assert_true(patterns.match(['xxx', 'X']))
        assert_true(patterns.match(['a', 'b']))
        assert_false(patterns.match(['a', 'b', 'c']))
        assert_true(patterns.match(['a']))
        assert_true(patterns.match(['B', 'XXX']))
        assert_false(patterns.match(['b', 'c']))
        assert_false(patterns.match(['c']))
        assert_true(patterns.match(['x', 'y', '321']))
        assert_false(patterns.match(['x', 'y', '32']))
        assert_false(patterns.match(['1', '2', '3', '4']))
        assert_true(patterns.match(['1', '22', '333']))

    def test_multiple_nots(self):
        patterns = TagPatterns(['xNOTyNOTz', '1 NOT 2 NOT 3 NOT 4'])
        assert_true(patterns.match(['x']))
        assert_false(patterns.match(['x', 'y']))
        assert_false(patterns.match(['x', 'z']))
        assert_false(patterns.match(['x', 'y', 'z']))
        assert_false(patterns.match(['xxx']))
        assert_true(patterns.match(['1']))
        assert_false(patterns.match(['1', '3', '4']))
        assert_false(patterns.match(['1', '2', '3']))
        assert_false(patterns.match(['1', '2', '3', '4']))

    def test_multiple_nots_with_ands(self):
        patterns = TagPatterns('a AND b NOT c AND d NOT e AND f')
        assert_true(patterns.match(['a', 'b']))
        assert_true(patterns.match(['a', 'b', 'c']))
        assert_true(patterns.match(['a', 'b', 'c', 'e']))
        assert_false(patterns.match(['a', 'b', 'c', 'd']))
        assert_false(patterns.match(['a', 'b', 'e', 'f']))
        assert_false(patterns.match(['a', 'b', 'c', 'd', 'e', 'f']))
        assert_false(patterns.match(['a', 'b', 'c', 'd', 'e']))

    def test_multiple_nots_with_ors(self):
        patterns = TagPatterns('a OR b NOT c OR d NOT e OR f')
        assert_true(patterns.match(['a']))
        assert_true(patterns.match(['B']))
        assert_false(patterns.match(['c']))
        assert_true(all(not patterns.match(['a', 'b', c]) for c in 'cdef'))
        assert_true(patterns.match(['a', 'x']))

    def test_starts_with_not(self):
        patterns = TagPatterns('NOTe')
        assert_true(patterns.match('d'))
        assert_false(patterns.match('e'))
        patterns = TagPatterns('NOT e OR f')
        assert_true(patterns.match('d'))
        assert_false(patterns.match('e'))
        assert_false(patterns.match('f'))

    def test_str(self):
        for pattern in ['a', 'NOT a', 'a NOT b', 'a AND b', 'a OR b', 'a*',
                        'a OR b NOT c OR d AND e OR ??']:
            assert_equal(str(TagPatterns(pattern)),
                         '[%s]' % pattern)
            assert_equal(str(TagPatterns(pattern.replace(' ', ''))),
                         '[%s]' % pattern)
            assert_equal(str(TagPatterns([pattern, 'x', pattern, 'y'])),
                         '[%s, x, y]' % pattern)

    def test_unicode(self):
        pattern = '\xe4 OR \xe5 NOT \xe6 AND \u2603 OR ??'
        expected = '[%s]' % pattern
        assert_equal(str(TagPatterns(pattern)), expected)
        assert_equal(str(TagPatterns(pattern.replace(' ', ''))), expected)

    def test_seq2str(self):
        patterns = TagPatterns(['is\xe4', '\xe4iti'])
        assert_equal(seq2str(patterns), "'is\xe4' and '\xe4iti'")


class AndOrPatternGenerator:
    tags = ['0', '1']
    operators = ['OR', 'AND']

    def __init__(self, max_length):
        self.max_length = max_length

    def __iter__(self):
        for tag in self.tags:
            for pattern in self._generate([tag], self.max_length-1):
                yield pattern

    def _generate(self, tokens, length):
        yield ' '.join(tokens)
        if length:
            for operator in self.operators:
                for tag in self.tags:
                    for pattern in self._generate(tokens + [operator, tag],
                                                  length-1):
                        yield pattern


if __name__ == '__main__':
    unittest.main()



import platform

# We assume that prompt is PS1='\u@\h \W \$ '
HOST = 'localhost'
USERNAME = 'test'
PASSWORD = 'test'
PROMPT = '$ '
FULL_PROMPT = '%s@%s ~ $ ' % (USERNAME, platform.uname()[1])
PROMPT_START = '%s@' % USERNAME
HOME = '/home/%s' % USERNAME



import time
import random
import string
import unittest

from robot.reporting.stringcache import StringCache, StringIndex
from robot.utils.asserts import assert_equal, assert_true, assert_false


try:
    long
except NameError:
    long = int


class TestStringCache(unittest.TestCase):

    def setUp(self):
        # To make test reproducable log the random seed if test fails
        self._seed = long(time.time() * 256)
        random.seed(self._seed)
        self.cache = StringCache()

    def _verify_text(self, string, expected):
        self.cache.add(string)
        assert_equal(('*', expected), self.cache.dump())

    def _compress(self, text):
        return self.cache._encode(text)

    def test_short_test_is_not_compressed(self):
        self._verify_text('short', '*short')

    def test_long_test_is_compressed(self):
        long_string = 'long'*1000
        self._verify_text(long_string, self._compress(long_string))

    def test_coded_string_is_at_most_1_characters_longer_than_raw(self):
        for i in range(300):
            id = self.cache.add(self._generate_random_string(i))
            assert_true(i+1 >= len(self.cache.dump()[id]),
                        'len(self._text_cache.dump()[id]) (%s) > i+1 (%s) [test seed = %s]'
                        % (len(self.cache.dump()[id]), i+1, self._seed))

    def test_long_random_strings_are_compressed(self):
        for i in range(30):
            value = self._generate_random_string(300)
            id = self.cache.add(value)
            assert_equal(self._compress(value), self.cache.dump()[id],
                          msg='Did not compress [test seed = %s]' % self._seed)

    def _generate_random_string(self, length):
        return ''.join(random.choice(string.digits) for _ in range(length))

    def test_indices_reused_instances(self):
        strings = ['', 'short', 'long'*1000, '']
        indices1 = [self.cache.add(s) for s in strings]
        indices2 = [self.cache.add(s) for s in strings]
        for i1, i2 in zip(indices1, indices2):
            assert_true(i1 is i2, 'not same: %s and %s' % (i1, i2))


class TestStringIndex(unittest.TestCase):

    def test_to_string(self):
        value = StringIndex(42)
        assert_equal(str(value), '42')

    def test_truth(self):
        assert_true(StringIndex(1))
        assert_true(StringIndex(-42))
        assert_false(StringIndex(0))


if __name__ == '__main__':
    unittest.main()



from robot.libraries.BuiltIn import BuiltIn, register_run_keyword

class DynamicRegisteredLibrary:
    
    def get_keyword_names(self):
        return ['dynamic_run_keyword']
    
    def run_keyword(self, name, args):
        dynamic_run_keyword(*args)

def dynamic_run_keyword(name, *args):
    return BuiltIn().run_keyword(name, *args)

register_run_keyword('DynamicRegisteredLibrary', 'dynamic_run_keyword', 1)



import os

DICT_FROM_VAR_FILE = dict(a='1', b=2, c='3')
ESCAPED_FROM_VAR_FILE = {'${a}': 'c:\\temp',
                         'b': '${2}',
                         os.sep: '\n' if os.sep == '/' else '\r\n',
                         '4=5\\=6': 'value'}


class ClassFromVarFile:
    attribute = DICT_FROM_VAR_FILE

    def get_escaped(self):
        return ESCAPED_FROM_VAR_FILE


OBJECT_FROM_VAR_FILE = ClassFromVarFile()



import json
import os
import tempfile
import unittest
from pathlib import Path

from jsonschema import Draft202012Validator

from robot.utils import PY_VERSION
from robot.utils.asserts import assert_equal
from robot.libdocpkg import LibraryDocumentation
from robot.libdocpkg.model import LibraryDoc, KeywordDoc
from robot.libdocpkg.htmlutils import HtmlToText, DocToHtml

get_shortdoc = HtmlToText().get_shortdoc_from_html
get_text = HtmlToText().html_to_plain_text

CURDIR = Path(__file__).resolve().parent
DATADIR = (CURDIR / '../../atest/testdata/libdoc/').resolve()
TEMPDIR = Path(os.getenv('TEMPDIR') or tempfile.gettempdir())
VALIDATOR = Draft202012Validator(
    json.loads((CURDIR / '../../doc/schema/libdoc.json').read_text())
)

try:
    from typing_extensions import TypedDict
except ImportError:
    TYPEDDICT_SUPPORTS_REQUIRED_KEYS = PY_VERSION >= (3, 9)
else:
    TYPEDDICT_SUPPORTS_REQUIRED_KEYS = True


def verify_shortdoc_output(doc_input, expected):
    current = get_shortdoc(doc_input)
    assert_equal(current, expected)


def verify_keyword_shortdoc(doc_format, doc_input, expected):
    libdoc = LibraryDoc(doc_format=doc_format)
    libdoc.keywords = [KeywordDoc(doc=doc_input)]
    assert_equal(libdoc.keywords[0].shortdoc, expected)


def run_libdoc_and_validate_json(filename):
    library = DATADIR / filename
    json_spec = LibraryDocumentation(library).to_json()
    VALIDATOR.validate(instance=json.loads(json_spec))


class TestHtmlToDoc(unittest.TestCase):

    def test_shortdoc_firstline(self):
        doc = """<p>This is the first line</p>
        <p>This is the second one</p>"""
        exp = "This is the first line"
        verify_shortdoc_output(doc, exp)

    def test_shortdoc_replace_format(self):
        doc = "<p>This is <b>bold</b> or <i>italic</i> or <i><b>italicbold</b></i> and code.</p>"
        exp = "This is *bold* or _italic_ or _*italicbold*_ and code."
        verify_shortdoc_output(doc, exp)

    def test_shortdoc_replace_format_multiline(self):
        doc = """<p>This is <b>bold</b>
        or <i>italic</i> or <i><b>italic
        bold</b></i> and <code>code</code>.</p>"""
        exp = """This is *bold*
        or _italic_ or _*italic
        bold*_ and ``code``."""
        verify_shortdoc_output(doc, exp)

    def test_shortdoc_unexcape_html(self):
        doc = """<p>This &amp; &quot;<b>&lt;b&gt;is&lt;/b&gt;</b>&quot;
        &lt;i&gt;the&lt;/i&gt; &lt;/p&gt;&apos;first&apos; line</p>"""
        exp = """This & "*<b>is</b>*"
        <i>the</i> </p>'first' line"""
        verify_shortdoc_output(doc, exp)


class TestKeywordShortDoc(unittest.TestCase):

    def test_shortdoc_with_multiline_plain_text(self):
        doc = """Writes the message to the console.

    If the ``newline`` argument is ``True``, a newline character is
    automatically added to the message.

    By default the message is written to the standard output stream.
    Using the standard error stream is possibly by giving the ``stream``
    argument value ``'stderr'``."""
        exp = "Writes the message to the console."
        verify_keyword_shortdoc('TEXT', doc, exp)

    def test_shortdoc_with_empty_plain_text(self):
        verify_keyword_shortdoc('TEXT', '', '')

    def test_shortdoc_with_multiline_robot_format(self):
        doc = """Writes the
*message* to
_the_ ``console``.

If the ``newline`` argument is ``True``, a newline character is
automatically added to the message.

By default the message is written to the standard output stream.
Using the standard error stream is possibly by giving the ``stream``
argument value ``'stderr'``."""
        exp = "Writes the *message* to _the_ ``console``."
        verify_keyword_shortdoc('ROBOT', doc, exp)

    def test_shortdoc_with_empty_robot_format(self):
        verify_keyword_shortdoc('ROBOT', '', '')

    def test_shortdoc_with_multiline_HTML_format(self):
        doc = """<p><strong>Writes</strong><br><em>the</em> <b>message</b>
to <i>the</i> <code>console</code>.<br><br>
If the <code>newline</code> argument is <code>True</code>, a newline character is
automatically added to the message.</p>
<p>By default the message is written to the standard output stream.
Using the standard error stream is possibly by giving the <code>stream</code>
argument value ``'stderr'``."""
        exp = "*Writes* _the_ *message* to _the_ ``console``."
        verify_keyword_shortdoc('HTML', doc, exp)

    def test_shortdoc_with_nonclosing_p_HTML_format(self):
        doc = """<p><strong>Writes</strong><br><em>the</em> <b>message</b>
to <i>the</i> <code>console</code>.<br><br>
If the <code>newline</code> argument is <code>True</code>, a newline character is
automatically added to the message.
<p>By default the message is written to the standard output stream.
Using the standard error stream is possibly by giving the <code>stream</code>
argument value ``'stderr'``."""
        exp = "*Writes* _the_ *message* to _the_ ``console``."
        verify_keyword_shortdoc('HTML', doc, exp)

    def test_shortdoc_with_empty_HTML_format(self):
        verify_keyword_shortdoc('HTML', '', '')

    def test_shortdoc_with_multiline_reST_format(self):
        doc = """Writes the **message**
to *the* console.

If the ``newline`` argument is ``True``, a newline character is
automatically added to the message.

By default the message is written to the standard output stream.
Using the standard error stream is possibly by giving the ``stream``
argument value ``'stderr'``."""
        exp = "Writes the **message** to *the* console."
        verify_keyword_shortdoc('REST', doc, exp)

    def test_shortdoc_with_empty_reST_format(self):
        verify_keyword_shortdoc('REST', '', '')


class TestLibdocJsonWriter(unittest.TestCase):

    def test_Annotations(self):
        run_libdoc_and_validate_json('Annotations.py')

    def test_Decorators(self):
        run_libdoc_and_validate_json('Decorators.py')

    def test_Deprecation(self):
        run_libdoc_and_validate_json('Deprecation.py')

    def test_DocFormat(self):
        run_libdoc_and_validate_json('DocFormat.py')

    def test_DynamicLibrary(self):
        run_libdoc_and_validate_json('DynamicLibrary.py::required')

    def test_DynamicLibraryWithoutGetKwArgsAndDoc(self):
        run_libdoc_and_validate_json('DynamicLibraryWithoutGetKwArgsAndDoc.py')

    def test_ExampleSpec(self):
        run_libdoc_and_validate_json('ExampleSpec.xml')

    def test_InternalLinking(self):
        run_libdoc_and_validate_json('InternalLinking.py')

    def test_KeywordOnlyArgs(self):
        run_libdoc_and_validate_json('KeywordOnlyArgs.py')

    def test_LibraryDecorator(self):
        run_libdoc_and_validate_json('LibraryDecorator.py')

    def test_module(self):
        run_libdoc_and_validate_json('module.py')

    def test_NewStyleNoInit(self):
        run_libdoc_and_validate_json('NewStyleNoInit.py')

    def test_no_arg_init(self):
        run_libdoc_and_validate_json('no_arg_init.py')

    def test_resource(self):
        run_libdoc_and_validate_json('resource.resource')

    def test_resource_with_robot_extension(self):
        run_libdoc_and_validate_json('resource.robot')

    def test_toc(self):
        run_libdoc_and_validate_json('toc.py')

    def test_TOCWithInitsAndKeywords(self):
        run_libdoc_and_validate_json('TOCWithInitsAndKeywords.py')

    def test_TypesViaKeywordDeco(self):
        run_libdoc_and_validate_json('TypesViaKeywordDeco.py')

    def test_DynamicLibrary_json(self):
        run_libdoc_and_validate_json('DynamicLibrary.json')

    def test_DataTypesLibrary_json(self):
        run_libdoc_and_validate_json('DataTypesLibrary.json')

    def test_DataTypesLibrary_xml(self):
        run_libdoc_and_validate_json('DataTypesLibrary.xml')

    def test_DataTypesLibrary_py(self):
        run_libdoc_and_validate_json('DataTypesLibrary.py')

    def test_DataTypesLibrary_libspex(self):
        run_libdoc_and_validate_json('DataTypesLibrary.libspec')


class TestJson(unittest.TestCase):

    def test_roundtrip(self):
        self._test('DynamicLibrary.json')

    def test_roundtrip_with_datatypes(self):
        self._test('DataTypesLibrary.json')

    def _test(self, lib):
        path = DATADIR / lib
        spec = LibraryDocumentation(path).to_json()
        data = json.loads(spec)
        with open(path) as f:
            orig_data = json.load(f)
        data['generated'] = orig_data['generated'] = None
        self.maxDiff = None
        self.assertDictEqual(data, orig_data)


class TestXmlSpec(unittest.TestCase):

    def test_roundtrip(self):
        self._test('DynamicLibrary.json')

    def test_roundtrip_with_datatypes(self):
        self._test('DataTypesLibrary.json')

    def _test(self, lib):
        path = TEMPDIR / 'libdoc-utest-spec.xml'
        orig_lib = LibraryDocumentation(DATADIR / lib)
        orig_lib.save(path, format='XML')
        spec_lib = LibraryDocumentation(path)
        orig_data = orig_lib.to_dictionary()
        spec_data = spec_lib.to_dictionary()
        orig_data['generated'] = spec_data['generated'] = None
        self.maxDiff = None
        self.assertDictEqual(orig_data, spec_data)


class TestLibdocTypedDictKeys(unittest.TestCase):

    def test_typed_dict_keys(self):
        library = DATADIR / 'DataTypesLibrary.py'
        spec = LibraryDocumentation(library).to_json()
        current_items = json.loads(spec)['dataTypes']['typedDicts'][0]['items']
        expected_items = [
            {
                "key": "longitude",
                "type": "float",
                "required": True if TYPEDDICT_SUPPORTS_REQUIRED_KEYS else None
            },
            {
                "key": "latitude",
                "type": "float",
                "required": True if TYPEDDICT_SUPPORTS_REQUIRED_KEYS else None
            },
            {
                "key": "accuracy",
                "type": "float",
                "required": False if TYPEDDICT_SUPPORTS_REQUIRED_KEYS else None
            }
        ]
        for exp_item in expected_items:
            for cur_item in current_items:
                if exp_item['key'] == cur_item['key']:
                    assert_equal(exp_item, cur_item)
                    break


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from io import BytesIO
from glob import glob
from os.path import join, exists, dirname, abspath
from subprocess import call
from urllib.request import urlopen
from zipfile import ZipFile
import os
import shutil


JASMINE_REPORTER_URL='https://github.com/larrymyers/jasmine-reporters/zipball/0.2.1'
BASE = abspath(dirname(__file__))
REPORT_DIR = join(BASE, 'jasmine-results')
EXT_LIB = join(BASE, '..', 'ext-lib')
JARDIR = join(EXT_LIB, 'jasmine-reporters', 'ext')


def run_tests():
    workdir = os.getcwd()
    os.chdir(BASE)
    download_jasmine_reporters()
    clear_reports()
    run()
    os.chdir(workdir)


def run():
    cmd = ['java', '-cp', '%s%s%s' % (join(JARDIR, 'js.jar'), os.pathsep, join(JARDIR, 'jline.jar')),
           'org.mozilla.javascript.tools.shell.Main',  '-opt', '-1', 'envjs.bootstrap.js',
           join(BASE, 'webcontent', 'SpecRunner.html')]
    call(cmd)


def clear_reports():
    if exists(REPORT_DIR):
        shutil.rmtree(REPORT_DIR)
    os.mkdir(REPORT_DIR)


def download_jasmine_reporters():
    if exists(join(EXT_LIB, 'jasmine-reporters')):
        return
    if not exists(EXT_LIB):
        os.mkdir(EXT_LIB)
    reporter = urlopen(JASMINE_REPORTER_URL)
    z = ZipFile(BytesIO(reporter.read()))
    z.extractall(EXT_LIB)
    extraction_dir = glob(join(EXT_LIB, 'larrymyers-jasmine-reporters*'))[0]
    print('Extracting Jasmine-Reporters to', extraction_dir)
    shutil.move(extraction_dir, join(EXT_LIB, 'jasmine-reporters'))


if __name__ == '__main__':
    run_tests()



import unittest

from robot.errors import DataError
from robot.utils.asserts import (assert_equal, assert_false,
                                 assert_raises_with_msg, assert_true)
from robot.variables.search import (search_variable, unescape_variable_syntax,
                                    VariableIterator)


class TestSearchVariable(unittest.TestCase):
    _identifiers = ['$', '@', '%', '&', '*']

    def test_empty(self):
        self._test('')
        self._test('                                       ')

    def test_no_vars(self):
        for inp in ['hello world', '$hello', '{hello}', r'$\{hello}',
                    '$h{ello}', 'a bit longer sting here']:
            self._test(inp)

    def test_not_string(self):
        self._test(42)
        self._test([1, 2, 3])

    def test_backslashes(self):
        for inp in ['\\', '\\\\', '\\\\\\\\\\', '\\hello\\\\world\\\\\\']:
            self._test(inp)

    def test_one_var(self):
        self._test('${hello}', '${hello}')
        self._test('1 @{hello} more', '@{hello}', start=2)
        self._test('*{hi}}', '*{hi}')
        self._test('{%{{hi}}', '%{{hi}}', start=1)
        self._test('-= ${} =-', '${}', start=3)

    def test_escape_internal_curlys(self):
        self._test(r'${embed:\d\{2\}}', r'${embed:\d\{2\}}')
        self._test(r'{}{${e:\d\{4\}-\d\{2\}-\d\{2\}}}}',
                   r'${e:\d\{4\}-\d\{2\}-\d\{2\}}', start=3)
        self._test(r'$&{\{\}\{\}\\}{}', r'&{\{\}\{\}\\}', start=1)
        self._test(r'${&{\}\{\\\\}\\}}{}', r'${&{\}\{\\\\}\\}')

    def test_matching_internal_curlys_dont_need_to_be_escaped(self):
        self._test(r'${embed:\d{2}}', r'${embed:\d{2}}')
        self._test(r'{}{${e:\d{4}-\d{2}-\d{2}}}}',
                   r'${e:\d{4}-\d{2}-\d{2}}', start=3)
        self._test(r'$&{{}{}\\}{}', r'&{{}{}\\}', start=1)
        self._test(r'${&{{\\\\}\\}}{}}', r'${&{{\\\\}\\}}')

    def test_uneven_curlys(self):
        for inp in ['${x', '${x:{}', '${y:{{}}', 'xx${z:{}xx', '{${{}{{}}{{',
                    r'${x\}', r'${x\\\}', r'${x\\\\\\\}']:
            for identifier in '$@&%':
                assert_raises_with_msg(
                    DataError,
                    "Variable '%s%s' was not closed properly."
                    % (identifier, inp.split('$')[1]),
                    search_variable, inp.replace('$', identifier)
                )
                self._test(inp.replace('$', identifier), ignore_errors=True)
        self._test('}{${xx:{}}}}}', '${xx:{}}', start=2)

    def test_escaped_uneven_curlys(self):
        self._test(r'${x:\{}', r'${x:\{}')
        self._test(r'${y:{\{}}', r'${y:{\{}}')
        self._test(r'xx${z:\{}xx', r'${z:\{}', start=2)
        self._test(r'{%{{}\{{}}{{', r'%{{}\{{}}', start=1)
        self._test(r'${xx:{}\}\}\}}', r'${xx:{}\}\}\}}')

    def test_multiple_vars(self):
        self._test('${hello} ${world}', '${hello}', 0)
        self._test('hi %{u}2 and @{u2} and also *{us3}', '%{u}', 3)
        self._test('0123456789 %{1} and @{2', '%{1}', 11)

    def test_escaped_var(self):
        self._test('\\${hello}')
        self._test('hi \\\\\\${hello} moi')

    def test_not_escaped_var(self):
        self._test('\\\\${hello}', '${hello}', 2)
        self._test('\\hi \\\\\\\\\\\\${hello} moi', '${hello}',
                   len('\\hi \\\\\\\\\\\\'))
        self._test('\\ ${hello}', '${hello}', 2)
        self._test('${hello}\\', '${hello}', 0)
        self._test('\\ \\ ${hel\\lo}\\', '${hel\\lo}', 4)

    def test_escaped_and_not_escaped_vars(self):
        for inp, var, start in [
                ('\\${esc} ${not}', '${not}', len('\\${esc} ')),
                ('\\\\\\${esc} \\\\${not}', '${not}',
                 len('\\\\\\${esc} \\\\')),
                ('\\${esc}\\\\${not}${n2}', '${not}', len('\\${esc}\\\\'))]:
            self._test(inp, var, start)

    def test_internal_vars(self):
        for inp, var, start in [
                ('${hello${hi}}', '${hello${hi}}', 0),
                ('bef ${${hi}hello} aft', '${${hi}hello}', 4),
                (r'\${not} ${hel${hi}lo} ', '${hel${hi}lo}', len(r'\${not} ')),
                ('${${hi}${hi}}\\', '${${hi}${hi}}', 0),
                ('${${hi${hi}}} ${xx}', '${${hi${hi}}}', 0),
                (r'${\${hi${hi}}}', r'${\${hi${hi}}}', 0),
                (r'\${${hi${hi}}}', '${hi${hi}}', len(r'\${')),
                (r'\${\${hi\\${h${i}}}}', '${h${i}}', len(r'\${\${hi\\'))]:
            self._test(inp, var, start)

    def test_incomplete_internal_vars(self):
        for inp in ['${var$', '${var${', '${var${int}']:
            for identifier in '$@&%':
                assert_raises_with_msg(
                    DataError,
                    "Variable '%s' was not closed properly."
                    % inp.replace('$', identifier),
                    search_variable, inp.replace('$', identifier)
                )
                self._test(inp.replace('$', identifier), ignore_errors=True)
        self._test('}{${xx:{}}}}}', '${xx:{}}', start=2)

    def test_item_access(self):
        self._test('${x}[0]', '${x}', items='0')
        self._test('.${x}[key]..', '${x}', start=1, items='key')
        self._test('${x}[]', '${x}', items='')
        self._test('${x}}[0]', '${x}')

    def test_nested_item_access(self):
        self._test('${x}[0][1]', '${x}', items=['0', '1'])
        self._test('xx${x}[key][42][-1][xxx]', '${x}', start=2,
                   items=['key', '42', '-1', 'xxx'])

    def test_item_access_with_vars(self):
        self._test('${x}[${i}]', '${x}', items='${i}')
        self._test('xx ${x}[${i}] ${xyz}', '${x}', start=3, items='${i}')
        self._test('$$$$${XX}[${${i}-${${${i}}}}]', '${XX}', start=4,
                   items='${${i}-${${${i}}}}')
        self._test('${${i}}[${j{}}]', '${${i}}', items='${j{}}')
        self._test('${x}[${i}][${k}]', '${x}', items=['${i}', '${k}'])

    def test_item_access_with_escaped_squares(self):
        self._test(r'${x}[\]]', '${x}', items=r'\]')
        self._test(r'${x}[\\]]', '${x}', items=r'\\')
        self._test(r'${x}[\[]', '${x}', items=r'\[')
        self._test(r'${x}\[k]', '${x}')
        self._test(r'${x}\[k', '${x}')
        self._test(r'${x}[k]\[i]', '${x}', items='k')

    def test_item_access_with_matching_squares(self):
        self._test('${x}[[]]', '${x}', items=['[]'])
        self._test('${x}[${y}[0][key]]', '${x}', items=['${y}[0][key]'])
        self._test('${x}[${y}[0]][key]', '${x}', items=['${y}[0]', 'key'])

    def test_unclosed_item(self):
        for inp in ['${x}[0', '${x}[0][key', r'${x}[0\]']:
            msg = "Variable item '%s' was not closed properly." % inp
            assert_raises_with_msg(DataError, msg, search_variable, inp)
            self._test(inp, ignore_errors=True)
        self._test('[${var}[i]][', '${var}', start=1, items='i')

    def test_nested_list_and_dict_item_syntax(self):
        self._test('@{x}[0]', '@{x}', items='0')
        self._test('&{x}[key]', '&{x}', items='key')

    def test_escape_item(self):
        self._test('${x}\\[0]', '${x}')
        self._test('@{x}\\[0]', '@{x}')
        self._test('&{x}\\[key]', '&{x}')

    def test_no_item_with_others_vars(self):
        self._test('%{x}[0]', '%{x}')
        self._test('*{x}[0]', '*{x}')

    def test_custom_identifiers(self):
        for inp, start in [('@{x}${y}', 4),
                           ('%{x} ${y}', 5),
                           ('*{x}567890${y}', 10),
                           (r'&{x}%{x}@{x}\${x}${y}',
                            len(r'&{x}%{x}@{x}\${x}'))]:
            self._test(inp, '${y}', start, identifiers=['$'])

    def test_identifier_as_variable_name(self):
        for i in self._identifiers:
            for count in 1, 2, 3, 42:
                var = '%s{%s}' % (i, i*count)
                self._test(var, var)
                self._test(var+'spam', var)
                self._test('eggs'+var+'spam', var, start=4)
                self._test(i+var+i, var, start=1)

    def test_identifier_as_variable_name_with_internal_vars(self):
        for i in self._identifiers:
            for count in 1, 2, 3, 42:
                var = '%s{%s{%s}}' % (i, i*count, i)
                self._test(var, var)
                self._test('eggs'+var+'spam', var, start=4)
                var = '%s{%s{%s}}' % (i, i*count, i*count)
                self._test(var, var)
                self._test('eggs'+var+'spam', var, start=4)

    def test_many_possible_starts_and_ends(self):
        self._test('{}'*10000)
        self._test('{{}}'*1000 + '${var}', '${var}', start=4000)
        self._test('${var}' + '[i]'*1000, '${var}', items=['i']*1000)

    def test_complex(self):
        self._test('${${PER}SON${2}[${i}]}', '${${PER}SON${2}[${i}]}')
        self._test('${x}[${${PER}SON${2}[${i}]}]', '${x}',
                   items='${${PER}SON${2}[${i}]}')

    def _test(self, inp, variable=None, start=0, items=None,
              identifiers=_identifiers, ignore_errors=False):
        if isinstance(items, str):
            items = (items,)
        elif items is None:
            items = ()
        else:
            items = tuple(items)
        if variable is None or ignore_errors:
            identifier = base = None
            start = end = -1
            is_var = is_scal_var = is_list_var = is_dict_var = False
        else:
            identifier = variable[0]
            base = variable[2:-1]
            end = start + len(variable)
            is_var = inp == variable
            if items:
                items_str = ''.join('[%s]' % i for i in items)
                end += len(items_str)
                is_var = inp == '%s%s' % (variable, items_str)
            is_list_var = is_var and inp[0] == '@'
            is_dict_var = is_var and inp[0] == '&'
            is_scal_var = is_var and inp[0] == '$'
        match = search_variable(inp, identifiers, ignore_errors)
        assert_equal(match.base, base, '%r base' % inp)
        assert_equal(match.start, start, '%r start' % inp)
        assert_equal(match.end, end, '%r end' % inp)
        assert_equal(match.before, inp[:start] if start != -1 else inp)
        assert_equal(match.match, inp[start:end] if end != -1 else None)
        assert_equal(match.after, inp[end:] if end != -1 else None)
        assert_equal(match.identifier, identifier, '%r identifier' % inp)
        assert_equal(match.items, items, '%r item' % inp)
        assert_equal(match.is_variable(), is_var)
        assert_equal(match.is_scalar_variable(), is_scal_var)
        assert_equal(match.is_list_variable(), is_list_var)
        assert_equal(match.is_dict_variable(), is_dict_var)

    def test_is_variable(self):
        for no in ['', 'xxx', '${var} not alone', r'\${notvar}', r'\\${var}',
                   '${var}xx}', '${x}${y}']:
            assert_false(search_variable(no).is_variable(), no)
        for yes in ['${var}', r'${var$\{}', '${var${internal}}', '@{var}',
                    '@{var}[0]']:
            assert_true(search_variable(yes).is_variable(), yes)

    def test_is_list_variable(self):
        for no in ['', 'xxx', '@{var} not alone', r'\@{notvar}', r'\\@{var}',
                   '@{var}xx}', '@{x}@{y}', '${scalar}', '&{dict}']:
            assert_false(search_variable(no).is_list_variable())
        assert_true(search_variable('@{list}').is_list_variable())
        assert_true(search_variable('@{x}[0]').is_list_variable())
        assert_true(search_variable('@{grandpa}[mother][child]').is_list_variable())

    def test_is_dict_variable(self):
        for no in ['', 'xxx', '&{var} not alone', r'\@{notvar}', r'\\&{var}',
                   '&{var}xx}', '&{x}&{y}', '${scalar}', '@{list}']:
            assert_false(search_variable(no).is_dict_variable())
        assert_true(search_variable('&{dict}').is_dict_variable())
        assert_true(search_variable('&{yzy}[afa]').is_dict_variable())
        assert_true(search_variable('&{x}[k][foo][bar][1]').is_dict_variable())


class TestVariableIterator(unittest.TestCase):

    def test_no_variables(self):
        iterator = VariableIterator('no vars here', identifiers='$')
        assert_equal(list(iterator), [])
        assert_equal(bool(iterator), False)
        assert_equal(len(iterator), 0)

    def test_one_variable(self):
        iterator = VariableIterator('one ${var} here', identifiers='$')
        assert_equal(list(iterator), [('one ', '${var}', ' here')])
        assert_equal(bool(iterator), True)
        assert_equal(len(iterator), 1)

    def test_multiple_variables(self):
        iterator = VariableIterator('${1} @{2} and %{3}', identifiers='$@%')
        assert_equal(list(iterator), [('', '${1}', ' @{2} and %{3}'),
                                      (' ', '@{2}', ' and %{3}'),
                                      (' and ', '%{3}', '')])
        assert_equal(bool(iterator), True)
        assert_equal(len(iterator), 3)

    def test_can_be_iterated_many_times(self):
        iterator = VariableIterator('one ${var} here', identifiers='$')
        assert_equal(list(iterator), [('one ', '${var}', ' here')])
        assert_equal(list(iterator), [('one ', '${var}', ' here')])
        assert_equal(bool(iterator), True)
        assert_equal(bool(iterator), True)
        assert_equal(len(iterator), 1)
        assert_equal(len(iterator), 1)


class TestUnescapeVariableSyntax(unittest.TestCase):

    def test_no_backslash(self):
        for inp in ['no escapes', '']:
            self._test(inp)

    def test_no_variable(self):
        for inp in ['\\', r'\n', r'\d+', r'\u2603', r'\$', r'\@', r'\&']:
            self._test(inp)
            self._test('Hello, %s!' % inp)

    def test_unescape_variable(self):
        for i in '$@&%':
            self._test(r'\%s{var}' % i, '%s{var}' % i)
            self._test(r'=\%s{var}=' % i, '=%s{var}=' % i)
            self._test(r'\\%s{var}' % i)
            self._test(r'\\\%s{var}' % i, r'\\%s{var}' % i)
            self._test(r'\\\\%s{var}' % i)
        self._test(r'\${1} \@{2} \&{3} \%{4}', '${1} @{2} &{3} %{4}')

    def test_unescape_curlies(self):
        self._test(r'\{', '{')
        self._test(r'\}', '}')
        self._test(r'=\}=\{=', '=}={=')
        self._test(r'=\\}=\\{=')
        self._test(r'=\\\}=\\\{=', r'=\\}=\\{=')
        self._test(r'=\\\\}=\\\\{=')

    def test_misc(self):
        self._test(r'$\{foo\}', '${foo}')
        self._test(r'\$\{foo\}', r'\${foo}')
        self._test(r'\${\n}', r'${\n}')
        self._test(r'\${\${x}}', r'${${x}}')
        self._test(r'\${foo', r'\${foo')

    def _test(self, inp, exp=None):
        if exp is None:
            exp = inp
        assert_equal(unescape_variable_syntax(inp), exp)


if __name__ == '__main__':
    unittest.main()



from enum import Enum, IntEnum
from typing import Any, Dict, List, Optional, Union
try:
    from typing_extensions import TypedDict
except ImportError:
    from typing import TypedDict

from robot.api.deco import library


class HttpCredentials(TypedDict):
    username: str
    password: str


class _GeoCoordinated(TypedDict):
    longitude: float
    latitude: float


class GeoLocation(_GeoCoordinated, total=False):
    """Defines the geolocation.

    - ``latitude`` Latitude between -90 and 90.
    - ``longitude`` Longitude between -180 and 180.
    - ``accuracy`` *Optional* Non-negative accuracy value. Defaults to 0.

    Example usage: ``{'latitude': 59.95, 'longitude': 30.31667}``
    """
    accuracy: float


class Small(IntEnum):
    """This is the Documentation.

    This was defined within the class definition.
    """
    one = 1
    two = 2
    three = 3
    four = 4


AssertionOperator = Enum(
    "AssertionOperator",
    {
        "equal": "==",
        "==": "==",
        "<": "<",
        ">": ">",
        "<=": "<=",
        ">=": ">="
    },
)
AssertionOperator.__doc__ = """This is some Doc

This has was defined by assigning to __doc__."""


class CustomType:
    """This doc not used because converter method has doc."""
    @classmethod
    def parse(cls, value: Union[str, int]):
        """Converter method doc is used when defined."""
        return value


class CustomType2:
    """Class doc is used when converter method has no doc."""
    def __init__(self, value):
        self.value = value


class Unknown:
    pass


class A:
    @classmethod
    def not_used_converter_should_not_be_documented(cls, value):
        return 1


@library(converters={CustomType: CustomType.parse,
                     CustomType2: CustomType2,
                     A: A.not_used_converter_should_not_be_documented},
         auto_keywords=True)
class DataTypesLibrary:
    """This Library has Data Types.

    It has some in ``__init__`` and others in the `Keywords`.

    The DataTypes are the following that should be linked.
    `HttpCredentials` , `GeoLocation` , `Small` and `AssertionOperator`.
    """

    def __init__(self, credentials: Small = Small.one):
        """This is the init Docs.

        It links to `Set Location` keyword and to `GeoLocation` data type.
        """
        pass

    def set_location(self, location: GeoLocation):
        pass

    def assert_something(self, value, operator: Optional[AssertionOperator] = None, exp: str = 'something?'):
        """This links to `AssertionOperator` .

        This is the next Line that links to `Set Location` .
        """
        pass

    def funny_unions(self,
                     funny: Union[
                         bool,
                         Union[
                             int,
                             float,
                             bool,
                             str,
                             AssertionOperator,
                             Small,
                             GeoLocation,
                             None]] = AssertionOperator.equal):
        pass

    def typing_types(self, list_of_str: List[str], dict_str_int: Dict[str, int], whatever: Any, *args: List[Any]):
        pass

    def custom(self, arg: CustomType, arg2: 'CustomType2', arg3: CustomType, arg4: Unknown):
        pass



import os

outpath = os.path.join(os.getenv('TEMPDIR'), 'listen_by_module.txt')
OUTFILE = open(outpath, 'w')
ROBOT_LISTENER_API_VERSION = 2


def start_suite(name, attrs):
    metastr = ' '.join('%s: %s' % (k, v) for k, v in attrs['metadata'].items())
    OUTFILE.write("SUITE START: %s (%s) '%s' [%s]\n"
                  % (name, attrs['id'], attrs['doc'], metastr))

def start_test(name, attrs):
    tags = [str(tag) for tag in attrs['tags']]
    OUTFILE.write("TEST START: %s (%s, line %s) '%s' %s\n"
                  % (name, attrs['id'], attrs['lineno'], attrs['doc'],
                     tags))

def start_keyword(name, attrs):
    if attrs['assign']:
        assign = '%s = ' % ', '.join(attrs['assign'])
    else:
        assign = ''
    name = name + ' ' if name else ''
    if attrs['args']:
        args = '%s ' % [str(a) for a in attrs['args']]
    else:
        args = ''
    OUTFILE.write("%s START: %s%s%s(line %d)\n"
                  % (attrs['type'], assign, name, args, attrs['lineno']))

def log_message(message):
    msg, level = message['message'], message['level']
    if level != 'TRACE' and 'Traceback' not in msg:
        OUTFILE.write('LOG MESSAGE: [%s] %s\n' % (level, msg))

def message(message):
    msg, level = message['message'], message['level']
    if 'Settings' in msg:
        OUTFILE.write('Got settings on level: %s\n' % level)

def end_keyword(name, attrs):
    kw_type = 'KW' if attrs['type'] == 'Keyword' else attrs['type'].upper()
    OUTFILE.write("%s END: %s\n" % (kw_type, attrs['status']))

def end_test(name, attrs):
    if attrs['status'] == 'PASS':
        OUTFILE.write('TEST END: PASS\n')
    else:
        OUTFILE.write("TEST END: %s %s\n"
                      % (attrs['status'], attrs['message']))

def end_suite(name, attrs):
    OUTFILE.write('SUITE END: %s %s\n' % (attrs['status'], attrs['statistics']))

def output_file(path):
    _out_file('Output', path)

def report_file(path):
    _out_file('Report', path)

def log_file(path):
    _out_file('Log', path)

def debug_file(path):
    _out_file('Debug', path)

def _out_file(name, path):
    assert os.path.isabs(path)
    OUTFILE.write('%s: %s\n' % (name, os.path.basename(path)))

def close():
    OUTFILE.write('Closing...\n')
    OUTFILE.close()



class HelperLib:

    def __init__(self, name, doc, metadata, source, options):
        self.name = name
        self.doc = doc
        self.metadata = metadata
        self.source = source
        self.options = options

    def import_time_value_should_be(self, name, expected):
        actual = getattr(self, name)
        if not isinstance(actual, str):
            expected = eval(expected)
        if actual != expected:
            raise AssertionError(f'{actual} != {expected}')



class name:

    def get_keyword_names(self):
        return ['No dots in keyword name in library with dots in name',
                'Dots.in.name.in.a.library.with.dots.in.name',
                'Multiple...dots . . in . a............row.in.a.library.with.dots.in.name',
                'Ending with a dot. In a library with dots in name.',
                'Conflict']

    def run_keyword(self, name, args):
        print("Running keyword '%s'." % name)
        return '-'.join(args)



class ObjectToReturn:

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name

    def exception(self, name, msg=""):
        try:
            exception = getattr(__builtins__, name)
        except AttributeError:  # __builtins__ is sometimes a dict, go figure
            exception = __builtins__[name]
        raise exception(msg)



from os.path import join, abspath

__all__ = ['join_with_execdir', 'abspath', 'attr_is_not_kw',
           '_not_kw_even_if_listed_in_all', 'extra stuff', None]

def join_with_execdir(arg):
    return join(abspath('.'), arg)

def not_in_all():
    pass

attr_is_not_kw = 'Listed in __all__ but not a fuction'

def _not_kw_even_if_listed_in_all():
    print('Listed in __all__ but starts with an underscore')



import unittest

from robot.utils.asserts import assert_equal, assert_true, assert_false

from robot.output.logger import Logger
from robot.output.console.verbose import VerboseOutput


class MessageMock:

    def __init__(self, timestamp, level, message):
        self.timestamp = timestamp
        self.level = level
        self.message = message


class LoggerMock:

    def __init__(self, *expected):
        self.expected = list(expected)
        self.logger = self

    def message(self, msg):
        exp_msg, exp_level = self.expected.pop(0)
        assert_equal(msg.level, exp_level)
        assert_equal(msg.message, exp_msg)
        self.msg = msg

    def copy(self):
        return LoggerMock(*self.expected)


class LoggerMock2(LoggerMock):

    def output_file(self, name, path):
        self.output_file = (name, path)

    def close(self):
        self.closed = True


class TestLogger(unittest.TestCase):

    def setUp(self):
        self.logger = Logger(register_console_logger=False)

    def test_write_to_one_logger(self):
        logger = LoggerMock(('Hello, world!', 'INFO'))
        self.logger.register_logger(logger)
        self.logger.write('Hello, world!', 'INFO')
        assert_true(logger.msg.timestamp.startswith('20'))

    def test_write_to_one_logger_with_trace_level(self):
        logger = LoggerMock(('expected message', 'TRACE'))
        self.logger.register_logger(logger)
        self.logger.write('expected message', 'TRACE')
        assert_true(hasattr(logger, 'msg'))

    def test_write_to_multiple_loggers(self):
        logger = LoggerMock(('Hello, world!', 'INFO'))
        logger2 = logger.copy()
        logger3 = logger.copy()
        self.logger.register_logger(logger, logger2, logger3)
        self.logger.message(MessageMock('', 'INFO', 'Hello, world!'))
        assert_true(logger.msg is logger2.msg)
        assert_true(logger.msg is logger.msg)

    def test_write_multiple_messages(self):
        msgs = [('0', 'ERROR'), ('1', 'WARN'), ('2', 'INFO'), ('3', 'DEBUG'), ('4', 'TRACE')]
        logger = LoggerMock(*msgs)
        self.logger.register_logger(logger)
        for msg, level in msgs:
            self.logger.write(msg, level)
            assert_equal(logger.msg.message, msg)
            assert_equal(logger.msg.level, level)

    def test_all_methods(self):
        logger = LoggerMock2(('Hello, world!', 'INFO'))
        self.logger.register_logger(logger)
        self.logger.output_file('name', 'path')
        self.logger.close()
        assert_equal(logger.output_file, ('name', 'path'))
        assert_true(logger.closed)

    def test_registered_logger_does_not_need_all_methods(self):
        logger = LoggerMock(('Hello, world!', 'INFO'))
        self.logger.register_logger(logger)
        self.logger.output_file('name', 'path')
        self.logger.close()

    def test_close_removes_registered_loggers(self):
        logger = LoggerMock(('Hello, world!', 'INFO'))
        logger2 = LoggerMock2(('Hello, world!', 'INFO'))
        self.logger.register_logger(logger, logger2)
        self.logger.close()
        assert_equal(self.logger._other_loggers, [])

    def test_registering_syslog_with_none_path_does_nothing(self):
        self.logger.register_syslog('None')
        assert_equal(self.logger._syslog, None)

    def test_cached_messages_are_given_to_registered_writers(self):
        self.logger.write('This is a cached message', 'INFO')
        self.logger.write('Another cached message', 'TRACE')
        logger = LoggerMock(('This is a cached message', 'INFO'),
                            ('Another cached message', 'TRACE'))
        self.logger.register_logger(logger)
        assert_equal(logger.msg.message, 'Another cached message')

    def test_message_cache_can_be_turned_off(self):
        self.logger.disable_message_cache()
        self.logger.write('This message is not cached', 'INFO')
        logger = LoggerMock(('', ''))
        self.logger.register_logger(logger)
        assert_false(hasattr(logger, 'msg'))

    def test_start_and_end_suite_test_and_keyword(self):
        class MyLogger:
            def start_suite(self, suite): self.started_suite = suite
            def end_suite(self, suite): self.ended_suite = suite
            def start_test(self, test): self.started_test = test
            def end_test(self, test): self.ended_test = test
            def start_keyword(self, keyword): self.started_keyword = keyword
            def end_keyword(self, keyword): self.ended_keyword = keyword
        class Arg:
            type = None
        logger = MyLogger()
        self.logger.register_logger(logger)
        for name in 'suite', 'test', 'keyword':
            arg = Arg()
            arg.result = arg
            for stend in 'start', 'end':
                getattr(self.logger, stend + '_' + name)(arg)
                assert_equal(getattr(logger, stend + 'ed_' + name), arg)

    def test_verbose_console_output_is_automatically_registered(self):
        logger = Logger()
        start_suite = logger._console_logger.start_suite
        assert_true(start_suite.__self__.__class__ is VerboseOutput)

    def test_automatic_console_logger_can_be_disabled(self):
        logger = Logger()
        logger.unregister_console_logger()
        assert_equal(logger._console_logger, None)
        self._number_of_registered_loggers_should_be(0, logger)

    def test_automatic_console_logger_can_be_disabled_after_registering_logger(self):
        logger = Logger()
        mock = LoggerMock()
        logger.register_logger(mock)
        self._number_of_registered_loggers_should_be(2, logger)
        logger.unregister_console_logger()
        self._number_of_registered_loggers_should_be(1, logger)

    def test_disabling_automatic_logger_multiple_times_has_no_effect(self):
        logger = Logger()
        logger.unregister_console_logger()
        self._number_of_registered_loggers_should_be(0, logger)
        logger.unregister_console_logger()
        logger.unregister_console_logger()
        self._number_of_registered_loggers_should_be(0, logger)
        logger.register_logger(LoggerMock())
        logger.unregister_console_logger()
        self._number_of_registered_loggers_should_be(1, logger)

    def test_registering_console_logger_disables_automatic_console_logger(self):
        logger = Logger()
        logger.register_console_logger(width=42)
        self._number_of_registered_loggers_should_be(1, logger)
        assert_equal(logger._console_logger.start_suite.__self__._writer._width, 42)

    def test_unregister_logger(self):
        logger1, logger2, logger3 = LoggerMock(), LoggerMock(), LoggerMock()
        self.logger.register_logger(logger1, logger2, logger3)
        self.logger.unregister_logger(logger2)
        self._number_of_registered_loggers_should_be(2)
        self.logger.unregister_logger(logger3, logger1)
        self._number_of_registered_loggers_should_be(0)

    def test_unregistering_non_registered_logger_is_ok(self):
        logger1, logger2 = LoggerMock(), LoggerMock()
        self.logger.register_logger(logger1)
        self.logger.unregister_logger(logger2)
        self.logger.unregister_logger(None)

    def test_start_and_end_loggers_and_iter(self):
        logger = Logger()
        console = logger._console_logger.logger
        xml = LoggerMock()
        listener = LoggerMock()
        lib_listener = LoggerMock()
        other = LoggerMock()
        logger.register_xml_logger(xml)
        logger.register_listeners(listener, lib_listener)
        logger.register_logger(other)
        assert_equal([proxy.logger for proxy in logger.start_loggers],
                     [other, console, xml, listener, lib_listener])
        assert_equal([proxy.logger for proxy in logger.end_loggers],
                     [listener, lib_listener, console, xml, other])
        assert_equal(list(logger), list(logger.end_loggers))

    def _number_of_registered_loggers_should_be(self, number, logger=None):
        logger = logger or self.logger
        assert_equal(len(list(logger)), number)


if __name__ == "__main__":
    unittest.main()



from robot.api import ContinuableFailure, FatalError


class FatalCatastrophyException(RuntimeError):
    ROBOT_EXIT_ON_FAILURE = True


class ContinuableApocalypseException(RuntimeError):
    ROBOT_CONTINUE_ON_FAILURE = True


def exit_on_failure(msg='BANG!', standard=False, **config):
    exception = FatalError if standard else FatalCatastrophyException
    raise exception(msg, **config)


def raise_continuable_failure(msg='Can be continued', standard=False):
    exception = ContinuableFailure if standard else ContinuableApocalypseException
    raise exception(msg)


def value_error(msg):
    raise ValueError(msg)



import sys


def output(rc=0, stdout='', stderr='', count=1):
    if stdout:
        sys.stdout.write((stdout+'\n') * int(count))
    if stderr:
        sys.stderr.write((stderr+'\n') * int(count))
    return int(rc)


if __name__ == '__main__':
    rc = output(*sys.argv[1:])
    sys.exit(rc)



class NonAsciiKeywordNames:

    def __init__(self, include_latin1=False):
        self.names = ['Unicode nön-äscïï',
                      '\u2603',  # snowman
                      'UTF-8 nön-äscïï'.encode('UTF-8')]
        if include_latin1:
            self.names.append('Latin1 nön-äscïï'.encode('latin1'))

    def get_keyword_names(self):
        return self.names

    def run_keyword(self, name, args):
        return name



from robot.conf import Language


class Custom(Language):
    settings_header = 'H S'
    variables_header = 'H v'
    test_cases_header = 'h te'
    tasks_header = 'H Ta'
    keywords_header = 'H k'
    comments_header = 'h C'
    library_setting = 'L'
    resource_setting = 'R'
    variables_setting = 'V'
    name_setting = 'N'
    documentation_setting = 'D'
    metadata_setting = 'M'
    suite_setup_setting = 'S S'
    suite_teardown_setting = 'S T'
    test_setup_setting = 't s'
    task_setup_setting = 'ta s'
    test_teardown_setting = 'T tea'
    task_teardown_setting = 'TA tea'
    test_template_setting = 'T TEM'
    task_template_setting = 'TA TEM'
    test_timeout_setting = 't ti'
    task_timeout_setting = 'ta ti'
    test_tags_setting = 'T Ta'
    task_tags_setting = 'Ta Ta'
    keyword_tags_setting = 'K T'
    setup_setting = 'S'
    teardown_setting = 'TeA'
    template_setting = 'Tem'
    tags_setting = 'Ta'
    timeout_setting = 'ti'
    arguments_setting = 'A'
    given_prefix = set()
    when_prefix = set()
    then_prefix = set()
    and_prefix = set()
    but_prefix = set()



from robot.api.deco import keyword


class MyLibrary1:

    def keyword_only_in_library_1(self):
        print("Keyword from library 1")

    def keyword_in_both_libraries(self):
        print("Keyword from library 1")

    def keyword_in_all_resources_and_libraries(self):
        print("Keyword from library 1")

    def keyword_everywhere(self):
        print("Keyword from library 1")

    def keyword_in_tc_file_overrides_others(self):
        raise Exception("This keyword should not be called")

    def keyword_in_resource_overrides_libraries(self):
        raise Exception("This keyword should not be called")

    def comment(self):
        print("Overrides keyword from BuiltIn library")

    def copy_directory(self):
        print("Overrides keyword from OperatingSystem library")

    def no_operation(self):
        print("Overrides keyword from BuiltIn library")

    def method(self):
        print("My name was set using 'robot_name' attribute!")

    method.robot_name = "Name set using 'robot_name' attribute"

    @keyword("Name set using 'robot.api.deco.keyword' decorator")
    def name_set_in_method_signature(self):
        print("My name was set using 'robot.api.deco.keyword' decorator!")

    @keyword(name='Custom nön-ÄSCII name')
    def non_ascii_would_not_work_here(self):
        pass

    @keyword()
    def no_custom_name_given_1(self):
        pass

    @keyword
    def no_custom_name_given_2(self):
        pass

    @keyword('Add ${number:\d+} Copies Of ${product:\w+} To Cart')
    def add_copies_to_cart(self, num, thing):
        return num, thing

    @keyword
    def _i_start_with_an_underscore_and_i_am_ok(self):
        print("I'm marked with @keyword")

    @keyword("Function name can be whatever")
    def _(self):
        print('Real name set by @keyword')

    @keyword
    def __(self):
        print('This name reduces to an empty string and is invalid')

    @property
    def should_not_be_accessed(self):
        raise SystemExit("Shouldn't be accessed when inspecting library")

    @property
    def _should_not_be_accessed(self):
        raise SystemExit("Shouldn't be accessed when inspecting library")



import sys
from robot.api import logger


class InitLogging:
    called = 0

    def __init__(self):
        InitLogging.called += 1
        print('*WARN* Warning via stdout in init', self.called)
        print('Info via stderr in init', self.called, file=sys.stderr)
        logger.warn('Warning via API in init %d' % self.called)

    def keyword(self):
        pass




from functools import wraps


def decorator(f):
    @wraps(f)
    def wrapper(*args, **kws):
        return f(*args, **kws)
    return wrapper


class WrappedMethods:

    @decorator
    def wrapped_method(self):
        pass

    @decorator
    def wrapped_method_with_arguments(self, a, b=2):
        pass



class RunKeywordLibrary:
    ROBOT_LIBRARY_SCOPE = 'TESTCASE'

    def __init__(self):
        self.kw_names = ['Run Keyword That Passes', 'Run Keyword That Fails']

    def get_keyword_names(self):
        return self.kw_names

    def run_keyword(self, name, args):
        try:
            method = dict(zip(self.kw_names, [self._passes, self._fails]))[name]
        except KeyError:
            raise AttributeError
        return method(args)

    def _passes(self, args):
        for arg in args:
            print(arg, end=' ')
        return ', '.join(args)

    def _fails(self, args):
        if not args:
            raise AssertionError('Failure')
        raise AssertionError('Failure: %s' % ' '.join(args))


class GlobalRunKeywordLibrary(RunKeywordLibrary):
    ROBOT_LIBRARY_SCOPE = 'GLOBAL'


class RunKeywordButNoGetKeywordNamesLibrary:

    def run_keyword(self, *args):
        return ' '.join(args)

    def some_other_keyword(self, *args):
        return ' '.join(args)



class EmbeddedArgs:

    def get_keyword_names(self):
        return ['Add ${count} Copies Of ${item} To Cart']

    def run_keyword(self, name, args):
        assert name == 'Add ${count} Copies Of ${item} To Cart'
        return args



"""Module test library."""

from robot.api import deco


__version__ = '0.1-alpha'


def keyword(a1='d', *a2):
    """A keyword.

    See `get hello` for details.
    """
    pass


def get_hello():
    """Get hello.

    See `importing` for explanation of nothing
    and `introduction` for no more information.
    """
    return 'foo'


def non_string_defaults(a=1, b=True, c=(1, 2, None)):
    pass


def non_ascii_string_defaults(arg='hyvä'):
    pass


def non_ascii_bytes_defaults(arg=b'hyv\xe4'):
    pass


def multiline_doc_with_split_short_doc():
    """This is short doc.
    It can span multiple
    physical
    lines and contain *formatting*.

    This is body. It can naturally also
    contain multiple lines.

    And paragraphs.
    """


def non_ascii_doc():
    """Hyvää yötä.

    Спасибо!
    """


def non_ascii_doc_with_escapes():
    """Hyv\xE4\xE4 y\xF6t\xE4."""


@deco.keyword('Set Name Using Robot Name Attribute')
def name_set_in_method_signature(a, b, *args, **kwargs):
    """
    This makes sure that @deco.keyword decorated kws don't lose their signatures
    """
    pass


@deco.keyword('Takes ${embedded} ${args}')
def takes_embedded_args(a=1, b=2, c=3):
    """A keyword which uses embedded args."""
    pass


@deco.keyword(tags=['1', 1, 'one', 'yksi'])
def keyword_with_tags_1():
    pass


@deco.keyword('Keyword with tags 2', ('2', 2, 'two', 'kaksi'))
def setting_both_name_and_tags_by_decorator():
    pass


def keyword_with_tags_3():
    """Set tags in documentation.

    Tags: tag1, tag2
    """

def robot_espacers(arg=" robot  escapers\n\t\r  "):
    pass



class PythonVarArgsConstructor:
    
    def __init__(self, mandatory, *varargs):
        self.mandatory = mandatory
        self.varargs = varargs

    def get_args(self):
        return self.mandatory, ' '.join(self.varargs)




class ObjectWithSetItemCap:
    def __init__(self) -> None:
        self._dict = {}

    def clear(self):
        self._dict = {}

    def __setitem__(self, item, value):
        self._dict[item] = value

    def __getitem__(self, item):
        return self._dict[item]

    @property
    def container(self):
        return self._dict

class ObjectWithoutSetItemCap:
    def __init__(self) -> None:
        pass

OBJECT_WITH_SETITEM_CAP = ObjectWithSetItemCap()
OBJECT_WITHOUT_SETITEM_CAP = ObjectWithoutSetItemCap()



from robot.libraries.BuiltIn import BuiltIn


class StaticLibrary:

    def add_static_keyword(self, name):
        def f(x):
            """This doc for static"""
            return x
        setattr(self, name, f)
        BuiltIn().reload_library(self)



import unittest
from pathlib import Path

from robot.parsing.suitestructure import IncludedFiles
from robot.utils.asserts import assert_equal


class TestIncludedFiles(unittest.TestCase):

    def test_match_when_no_patterns(self):
        self._test_match()

    def test_match_name(self):
        self._test_match('match.robot')
        self._test_match('no_match.robot', match=False)

    def test_match_path(self):
        self._test_match(Path('match.robot').absolute())
        self._test_match(Path('no_match.robot').absolute(), match=False)

    def test_match_relative_path(self):
        self._test_match('test/match.robot', path='test/match.robot')

    def test_glob_name(self):
        self._test_match('*.robot')
        self._test_match('[mp]???h.robot')
        self._test_match('no_*.robot', match=False)

    def test_glob_path(self):
        self._test_match(Path('*.r?b?t').absolute())
        self._test_match(Path('../*/match.r?b?t').absolute())
        self._test_match(Path('../*/match.r?b?t'))
        self._test_match(Path('*/match.r?b?t'), path='test/match.robot')
        self._test_match(Path('no_*.robot').absolute(), match=False)

    def test_recursive_glob(self):
        self._test_match('x/**/match.robot', path='x/y/z/match.robot')
        self._test_match('x/*/match.robot', path='x/y/z/match.robot', match=False)

    def test_case_normalize(self):
        self._test_match('MATCH.robot')
        self._test_match(Path('match.robot').absolute(), path='MATCH.ROBOT')

    def test_sep_normalize(self):
        self._test_match(str(Path('match.robot').absolute()).replace('\\', '/'))

    def test_directories_are_recursive(self):
        self._test_match('.')
        self._test_match('test', path='test/match.robot')
        self._test_match('test', path='test/x/y/x/match.robot')
        self._test_match('*', path='test/match.robot')

    def _test_match(self, pattern=None, path='match.robot', match=True):
        patterns = [pattern] if pattern else []
        path = Path(path).absolute()
        assert_equal(IncludedFiles(patterns).match(path), match)
        if pattern:
            assert_equal(IncludedFiles(['no', 'match', pattern]).match(path), match)


if __name__ == '__main__':
    unittest.main()



import unittest

from robot.errors import DataError
from robot.variables import VariableAssignment
from robot.utils.asserts import assert_equal, assert_raises


class TestResolveAssignment(unittest.TestCase):

    def test_one_scalar(self):
        self._verify_valid(['${var}'])

    def test_multiple_scalars(self):
        self._verify_valid('${v1} ${v2} ${v3}'.split())

    def test_list(self):
        self._verify_valid(['@{list}'])

    def test_dict(self):
        self._verify_valid(['&{dict}'])

    def test_scalars_and_list(self):
        self._verify_valid('${v1} ${v2} @{list}'.split())
        self._verify_valid('@{list} ${v1} ${v2}'.split())
        self._verify_valid('${v1} @{list} ${v2}'.split())

    def test_equal_sign(self):
        self._verify_valid(['${var} ='])
        self._verify_valid('${v1} ${v2} @{list}='.split())

    def test_multiple_lists_fails(self):
        self._verify_invalid(['@{v1}', '@{v2}'])
        self._verify_invalid(['${v1}', '@{v2}', '@{v3}'])

    def test_dict_with_others_fails(self):
        self._verify_invalid(['&{v1}', '&{v2}'])
        self._verify_invalid(['${v1}', '&{v2}'])

    def test_equal_sign_in_wrong_place(self):
        self._verify_invalid(['${v1}=','${v2}'])
        self._verify_invalid(['${v1} =','@{v2} ='])

    def _verify_valid(self, assign):
        assignment = VariableAssignment(assign)
        assignment.validate_assignment()
        expected = [a.rstrip('= ') for a in assign]
        assert_equal(assignment.assignment, expected)

    def _verify_invalid(self, assign):
        assert_raises(DataError, VariableAssignment(assign).validate_assignment)


if __name__ == '__main__':
    unittest.main()



import sys
from xmlrpc.client import Binary

from remoteserver import DirectResultRemoteServer


class BinaryResult:

    def return_binary(self, *ordinals):
        return self._result(return_=self._binary(ordinals))

    def return_binary_list(self, *ordinals):
        return self._result(return_=[self._binary([o]) for o in ordinals])

    def return_binary_dict(self, **ordinals):
        ret = dict((k, self._binary([v])) for k, v in ordinals.items())
        return self._result(return_=ret)

    def return_nested_binary(self, *stuff, **more):
        ret_list = [self._binary([o]) for o in stuff]
        ret_dict = dict((k, self._binary([v])) for k, v in more.items())
        ret_dict['list'] = ret_list[:]
        ret_dict['dict'] = ret_dict.copy()
        ret_list.append(ret_dict)
        return self._result(return_=ret_list)

    def log_binary(self, *ordinals):
        return self._result(output=self._binary(ordinals))

    def fail_binary(self, *ordinals):
        return self._result(error=self._binary(ordinals, b'Error: '),
                            traceback=self._binary(ordinals, b'Traceback: '))

    def _binary(self, ordinals, extra=b''):
        return Binary(extra + bytes(int(o) for o in ordinals))

    def _result(self, return_='', output='', error='', traceback=''):
        return {'status': 'PASS' if not error else 'FAIL',
                'return': return_, 'output': output,
                'error': error, 'traceback': traceback}


if __name__ == '__main__':
    DirectResultRemoteServer(BinaryResult(), *sys.argv[1:])



"""Library to document and test correct default value escaping."""
from robot.libraries.BuiltIn import BuiltIn

b = BuiltIn()


def verify_backslash(current='c:\\windows\\system', expected='c:\\windows\\system'):
    b.should_be_equal(current, expected)


def verify_internalvariables(current='first ${sca${lar}}  @{list}[${4}]  &{dict.key}[2] some env %{${somename}} and a \\${backslash}[${key}]   ',
                             expected='first ${sca${lar}}  @{list}[${4}]  &{dict.key}[2] some env %{${somename}} and a \\${backslash}[${key}]   '):
    b.should_be_equal(current, expected)


def verify_line_break(current='Hello\n World!\r\n End...\\n', expected='Hello\n World!\r\n End...\\n'):
    b.should_be_equal(current, expected)


def verify_line_tab(current='Hello\tWorld!\t\t End\\t...', expected='Hello\tWorld!\t\t End\\t...'):
    b.should_be_equal(current, expected)


def verify_spaces(current='    Hello\tW   orld!\t  \t En d\\t... ', expected='    Hello\tW   orld!\t  \t En d\\t... '):
    b.should_be_equal(current, expected)


def verify_variables(current='first ${scalar} then @{list} and &{dict.key}[2] some env %{username} and a \\${backslash}   ',
                     expected='first ${scalar} then @{list} and &{dict.key}[2] some env %{username} and a \\${backslash}   '):
    b.should_be_equal(current, expected)


def verify_all(current='first ${scalar} \nthen\t @{list} and \\\\&{dict.key}[2] so   \\    me env %{username} and a \\${backslash}   ',
               expected='first ${scalar} \nthen\t @{list} and \\\\&{dict.key}[2] so   \\    me env %{username} and a \\${backslash}   '):
    b.should_be_equal(current, expected)



from robot.api import SkipExecution


class CustomSkipException(Exception):
    ROBOT_SKIP_EXECUTION = True


def skip_with_message(msg, html=False):
    raise SkipExecution(msg, html)


def skip_with_custom_exception():
    raise CustomSkipException("Skipped with custom exception.")



import os

from robot.libraries.BuiltIn import BuiltIn


class ListenSome:
    ROBOT_LISTENER_API_VERSION = '2'

    def __init__(self):
        outpath = os.path.join(os.getenv('TEMPDIR'), 'listen_some.txt')
        self.outfile = open(outpath, 'w')

    def startTest(self, name, attrs):
        self.outfile.write(name + '\n')

    def endSuite(self, name, attrs):
        self.outfile.write(attrs['statistics'] + '\n')

    def close(self):
        self.outfile.close()


class WithArgs:
    ROBOT_LISTENER_API_VERSION = '2'

    def __init__(self, arg1, arg2='default'):
        outpath = os.path.join(os.getenv('TEMPDIR'), 'listener_with_args.txt')
        with open(outpath, 'a') as outfile:
            outfile.write("I got arguments '%s' and '%s'\n" % (arg1, arg2))


class WithArgConversion:
    ROBOT_LISTENER_API_VERSION = '2'

    def __init__(self, integer: int, boolean=False):
        assert integer == 42
        assert boolean is True


class SuiteAndTestCounts:
    ROBOT_LISTENER_API_VERSION = '2'
    exp_data = {
        "Subsuites & Custom name for 📂 'subsuites2'":
            ([], ['Subsuites', "Custom name for 📂 'subsuites2'"], 5),
        'Subsuites':
            ([], ['Sub1', 'Sub2'], 2),
        'Sub1':
            (['SubSuite1 First'], [], 1),
        'Sub2':
            (['SubSuite2 First'], [], 1),
        "Custom name for 📂 'subsuites2'":
            ([], ['Sub.Suite.4', "Custom name for 📜 'subsuite3.robot'"], 3),
        "Custom name for 📜 'subsuite3.robot'":
            (['SubSuite3 First', 'SubSuite3 Second'], [], 2),
        'Sub.Suite.4':
            (['Test From Sub Suite 4'], [], 1)
    }

    def start_suite(self, name, attrs):
        data = attrs['tests'], attrs['suites'], attrs['totaltests']
        if data != self.exp_data[name]:
            raise AssertionError('Wrong tests or suites in %s: %s != %s.'
                                 % (name, self.exp_data[name], data))


class KeywordType:
    ROBOT_LISTENER_API_VERSION = '2'

    def start_keyword(self, name, attrs):
        expected = self._get_expected_type(**attrs)
        if attrs['type'] != expected:
            raise AssertionError("Wrong keyword type '%s', expected '%s'."
                                 % (attrs['type'], expected))

    def _get_expected_type(self, kwname, libname, args, source, lineno, **ignore):
        if ' IN ' in kwname:
            return 'FOR'
        if ' = ' in kwname:
            return 'ITERATION'
        if not args:
            if kwname in ("'IF' == 'WRONG'", '${i} == 9'):
                return 'IF'
            if kwname == "'ELSE IF' == 'ELSE IF'":
                return 'ELSE IF'
            if kwname == '':
                source = os.path.basename(source)
                if source == 'for_loops.robot':
                    return 'BREAK' if lineno == 14 else 'CONTINUE'
                return 'ELSE'
        expected = args[0] if libname == 'BuiltIn' else kwname
        return {'Suite Setup': 'SETUP', 'Suite Teardown': 'TEARDOWN',
                'Test Setup': 'SETUP', 'Test Teardown': 'TEARDOWN',
                'Keyword Teardown': 'TEARDOWN'}.get(expected, 'KEYWORD')

    end_keyword = start_keyword


class KeywordStatus:
    ROBOT_LISTENER_API_VERSION = '2'

    def start_keyword(self, name, attrs):
        self._validate_status(attrs, 'NOT SET')

    def end_keyword(self, name, attrs):
        run_status = 'FAIL' if attrs['kwname'] == 'Fail' else 'PASS'
        self._validate_status(attrs, run_status)

    def _validate_status(self, attrs, run_status):
        expected = 'NOT RUN' if self._not_run(attrs) else run_status
        if attrs['status'] != expected:
            raise AssertionError('Wrong keyword status %s, expected %s.'
                                 % (attrs['status'], expected))

    def _not_run(self, attrs):
        return attrs['type'] in ('IF', 'ELSE') or attrs['args'] == ['not going here']


class KeywordExecutingListener:
    ROBOT_LISTENER_API_VERSION = '2'

    def start_test(self, name, attrs):
        self._run_keyword('Start %s' % name)

    def end_test(self, name, attrs):
        self._run_keyword('End %s' % name)

    def _run_keyword(self, arg):
        BuiltIn().run_keyword('Log', arg)


class SuiteSource:
    ROBOT_LISTENER_API_VERSION = '2'

    def __init__(self):
        self._started = 0
        self._ended = 0

    def start_suite(self, name, attrs):
        self._started += 1
        self._test_source(name, attrs['source'])

    def end_suite(self, name, attrs):
        self._ended += 1
        self._test_source(name, attrs['source'])

    def _test_source(self, suite, source):
        default = os.path.isfile
        verifier = {'Root': lambda source: source == '',
                    'Subsuites': os.path.isdir}.get(suite, default)
        if (source and not os.path.isabs(source)) or not verifier(source):
            raise AssertionError("Suite '%s' has wrong source '%s'."
                                 % (suite, source))

    def close(self):
        if not (self._started == self._ended == 5):
            raise AssertionError("Wrong number of started (%d) or ended (%d) "
                                 "suites. Expected 5."
                                 % (self._started, self._ended))


class Messages:
    ROBOT_LISTENER_API_VERSION = '2'

    def __init__(self, path):
        self.output = open(path, 'w')

    def log_message(self, msg):
        self.output.write('%s: %s\n' % (msg['level'], msg['message']))

    def close(self):
        self.output.close()



ROBOT_LIBRARY_SCOPE = 'Test Suite'  # this should be ignored
__version__ = 'test'  # this should be used as version of this library


def passing():
    pass

def failing():
    raise AssertionError('This is a failing keyword from module library')

def logging():
    print('Hello from module library')
    print('*WARN* WARNING!')

def returning():
    return 'Hello from module library'

def argument(arg):
    assert arg == 'Hello', "Expected 'Hello', got '%s'" % arg

def many_arguments(arg1, arg2, arg3):
    assert arg1 == arg2 == arg3, ("All arguments should have been equal, got: "
                                  "%s, %s and %s") % (arg1, arg2, arg3)

def default_arguments(arg1, arg2='Hi', arg3='Hello'):
    many_arguments(arg1, arg2, arg3)

def variable_arguments(*args):
    return sum([int(arg) for arg in args])

attribute = 'This is not a keyword!'

class NotLibrary:

    def two_arguments(self, arg1, arg2):
        msg = "Arguments should have been unequal, both were '%s'" % arg1
        assert arg1 != arg2, msg

    def not_keyword(self):
        pass


notlib = NotLibrary()
two_arguments_from_class = notlib.two_arguments

lambda_keyword = lambda arg: int(arg) + 1
lambda_keyword_with_two_args = lambda x, y: int(x) / int(y)

def _not_keyword():
    pass

def module_library():
    return "It should be OK to have an attribute with same name as the module"

